import doctest
from insights.parsers import mssql_conf
from insights.tests import context_wrap

MSSQL_CONF = """
[sqlagent]
enabled = false

[EULA]
accepteula = Y

[memory]
memorylimitmb = 3328
""".strip()


def test_mssql_conf():
    conf = mssql_conf.MsSQLConf(context_wrap(MSSQL_CONF))
    assert conf.has_option('memory', 'memorylimitmb') is True
    assert conf.get('memory', 'memorylimitmb') == '3328'


def test_documentation():
    env = {'conf': mssql_conf.MsSQLConf(context_wrap(MSSQL_CONF))}
    failed_count, tests = doctest.testmod(mssql_conf, globs=env)
    assert failed_count == 0



"""
Foreman and Candlepin logs
==========================

Module for parsing the log files in foreman-debug archive

.. note::
    Please refer to its super-class :class:`insights.core.LogFileOutput` for
    usage information.

Parsers provided by this module:

CandlepinErrorLog - file ``sos_commands/foreman/foreman-debug/var/log/candlepin/error.log``
-------------------------------------------------------------------------------------------

CandlepinLog - file ``/var/log/candlepin/candlepin.log``
--------------------------------------------------------

ProductionLog - file ``/var/log/foreman/production.log``
--------------------------------------------------------

ProxyLog - file ``/var/log/foreman-proxy/proxy.log``
----------------------------------------------------

SatelliteLog - file ``/var/log/foreman-installer/satellite.log``
----------------------------------------------------------------

"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.foreman_proxy_log)
class ProxyLog(LogFileOutput):
    """Class for parsing ``foreman-proxy/proxy.log`` file."""
    time_format = {
        'standard': '%d/%b/%Y:%H:%M:%S',  # 31/May/2016:09:57:34
        'error': '%Y-%m-%dT%H:%M:%S.%f'  # 2016-05-31T09:57:35.884636
    }


@parser(Specs.foreman_satellite_log)
class SatelliteLog(LogFileOutput):
    """Class for parsing ``foreman-installer/satellite.log`` file."""
    pass


@parser(Specs.foreman_production_log)
class ProductionLog(LogFileOutput):
    """Class for parsing ``foreman/production.log`` file."""
    pass


@parser(Specs.candlepin_log)
class CandlepinLog(LogFileOutput):
    """Class for parsing ``candlepin/candlepin.log`` file."""
    pass


@parser(Specs.candlepin_error_log)
class CandlepinErrorLog(LogFileOutput):
    """
    Class for parsing ``candlepin/error.log`` file.

    Sample log contents::

        2016-09-07 13:56:49,001 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 14:07:33,735 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 14:09:55,173 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 15:20:33,796 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 15:27:34,367 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 16:49:24,650 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 18:07:53,688 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.sync.Importer - Conflicts occurred during import that were
        2016-09-07 18:07:53,690 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.sync.Importer - [DISTRIBUTOR_CONFLICT]
        2016-09-07 18:07:53,711 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.resource.OwnerResource - Recording import failure
        org.candlepin.sync.ImportConflictException: Owner has already imported from another subscription management application.

    Examples:
        >>> candlepin_log = shared[Candlepin_Error_Log]
        >>> candlepin_log.get('req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28')[0]['raw_message']
        '2016-09-07 18:07:53,688 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.sync.Importer - Conflicts occurred during import that were'

        >>> candlepin_log.get_after(datetime(2016, 9, 7, 16, 0, 0)[0]['raw_message']
        '2016-09-07 16:49:24,650 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname'
    """
    pass


@parser(Specs.foreman_ssl_access_ssl_log)
class ForemanSSLAccessLog(LogFileOutput):
    """Class for parsing ``var/log/httpd/foreman-ssl_access_ssl.log`` file.

    Sample log contents::

        10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/385e688f-43ad-41b2-9fc7-593942ddec78 HTTP/1.1" 200 10736 "-" "-"
        10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/status HTTP/1.1" 200 263 "-" "-"
        10.185.73.33 - 8a31cd915917666001591d6fb44602a7 [27/Mar/2017:13:34:52 -0400] "GET /pulp/repos/Acme_Inc/Library/RHEL7_Sat_Cap        sule_Servers/content/dist/rhel/server/7/7Server/x86_64/os/repodata/repomd.xml HTTP/1.1" 200 2018 "-" "urlgrabber/3.10 yum/3.4.3"
        10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/4f8a39d0-38b6-4663-8b7e-03368be4d3ab/owner HTTP/1.1" 200 5159 "-"
        10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/385e688f-43ad-41b2-9fc7-593942ddec78/compliance HTTP/1.1" 200 5527
        10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/4f8a39d0-38b6-4663-8b7e-03368be4d3ab HTTP/1.1" 200 10695 "-" "-"


    Examples:
        >>> foreman_ssl_acess_log = shared[ForemanSSLAccessLog]
        >>> foreman_ssl_acess_log.get('req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28')
    """
    time_format = '%d/%b/%Y:%H:%M:%S'



"""
Dmesg
=====

Combiner for Dmesg information. It uses the results of the following parsers (if they are present):
:class:`insights.parsers.dmesg.DmesgLineList`,
:class:`insights.parsers.dmesg_log.DmesgLog`

Typical output of the ``/var/log/dmesg`` file is::

[    0.000000] Initializing cgroup subsys cpu
[    0.000000] Linux version 3.10.0-862.el7.x86_64 (mockbuild@x86-034.build.eng.bos.redhat.com) \
(gcc version 4.8.5 20150623 (Red Hat 4.8.5-28) (GCC) ) #1 SMP Wed Mar 21 18:14:51 EDT 2018
[    2.090905] SELinux:  Completing initialization.
[    2.090907] SELinux:  Setting up existing superblocks.
[    2.099684] systemd[1]: Successfully loaded SELinux policy in 82.788ms.
[    2.117410] ip_tables: (C) 2000-2006 Netfilter Core Team
[    2.117429] systemd[1]: Inserted module 'ip_tables'
[    2.376551] systemd-journald[441]: Received request to flush runtime journal from PID 1
[    2.716874] cryptd: max_cpu_qlen set to 100
[    2.804152] AES CTR mode by8 optimization enabled

Typical output of the ``dmesg`` command is::

[    2.939498] [TTM] Initializing pool allocator
[    2.939502] [TTM] Initializing DMA pool allocator
[    2.940800] [drm] fb mappable at 0xFC000000
[    2.940947] fbcon: cirrusdrmfb (fb0) is primary device
[    2.957375] Console: switching to colour frame buffer device 128x48
[    2.959322] cirrus 0000:00:02.0: fb0: cirrusdrmfb frame buffer device
[    2.959334] [drm] Initialized cirrus 1.0.0 20110418 for 0000:00:02.0 on minor 0
[    3.062459] XFS (vda1): Ending clean mount
[    5.048484] ip6_tables: (C) 2000-2006 Netfilter Core Team
[    5.102434] Ebtables v2.0 registered


Examples:
    >>> dmesg.dmesg_cmd_available
    True
    >>> dmesg.dmesg_log_available
    True
    >>> dmesg.dmesg_log_wrapped
    False
"""

from insights.core.plugins import combiner
from insights.parsers.dmesg import DmesgLineList
from insights.parsers.dmesg_log import DmesgLog


@combiner([DmesgLineList, DmesgLog])
class Dmesg(object):
    """
    Combiner for ``dmesg`` command and ``/var/log/dmesg`` file.
    """

    def __init__(self, dmesg_cmd, dmesg_log):
        if dmesg_cmd is not None:
            self.dmesg_cmd_available = True
            self.dmesg_cmd = dmesg_cmd
            self.dmesg_cmd_wrapped = True if 'Linux version' not in dmesg_cmd else False
        else:
            self.dmesg_cmd_available = False

        if dmesg_log is not None:
            self.dmesg_log_available = True
            self.dmesg_log = dmesg_log
            self.dmesg_log_wrapped = True if 'Linux version' not in dmesg_log else False
        else:
            self.dmesg_log_available = False



from insights.formats import FormatterAdapter
from insights.formats.html import HtmlFormat


class SimpleHtmlFormat(HtmlFormat):
    """
    This class prints a html summary of rule hits. It should be used
    as a context manager and given an instance of an
    ``insights.core.dr.Broker``. ``dr.run`` should be called within the context
    using the same broker.

    Args:
        broker (Broker): the broker to watch and provide a summary about.
        stream (file-like): Output is written to stream. Defaults to sys.stdout.
    """

    TEMPLATE = """
<!doctype html>
<html lang="en">
  <head>
    <!-- Required meta tags -->
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

    <title>{{root}}</title>
    <style>
    a.pass {
      color: green;
    }
    a.fail {
      color: red;
    }
    a.source {
      color: orange;
    }
    .main {
      margin-top: 100px;
      margin-bottom: 100px;
      margin-right: 150px;
      margin-left: 80px;
    }
    </style>
  </head>
  <body>
  <div class="main">
  <h2 align="center">Analysis of {{root}}</h2>
  <h4 align="center">Performed at {{start_time}} UTC</h4>
    <section>
      <h2>System Information</h2>
        <pre>
        {%- for rule in rules.make_info %}
{{rule.body}}
        {%- endfor %}
        </pre>
    </section>
    <h2>Rule Results</h2>
    <nav>
      <ul>
        {%- for response_type, rule_group in rules.items() %}
        {%- if response_type != "make_info" %}
        {%- for rule in rule_group %}
        <ul>
          <a id="{{rule.id}}_top"
             class="{% if response_type !='make_pass' %}fail{% else %}pass{% endif %}"
             href="#{{rule.id}}">
          {{rule.name}}
          </a>
        </ul>
        {%- endfor %}
        {%- endif %}
        {%- endfor %}
      </ul>
    </nav>
    <section>
    <h2>Rule Result Details</h2>
        {%- for response_type, rule_group in rules.items() %}
        {%- if response_type != "make_info" %}
        {%- for rule in rule_group %}
        <article>
          <header>
          <a id="{{rule.id}}"
             class="{% if response_type !='make_pass' %}fail{% else %}pass{% endif %}"
             href="#{{rule.id}}_top">
          {{rule.name}}
          </a>
          </header>
          <p>
          <pre>
{{rule.body}}
          </pre>
          </p>

          <hr />
          <p>
Contributing Data:
          <ol>
          {%- for d in rule.datasources %}
          <li>{{d}}</li>
          {%- endfor %}
          </ol>
          </p>

          <hr />
          <p> Rule Source: <a class="source" href="file://{{rule.source_path}}">{{rule.source_path}}</a></p>

          <hr />
          <div>
          <p>
          Documentation:
          <div>
          <pre>
{{rule.mod_doc}}
{{rule.rule_doc}}
          </pre>
          </div>
          </p>
          </div>
          <hr />
        </article>
        {%- endfor %}
        {%- endif %}
        {%- endfor %}
    </section>
  </div>
  </body>
</html>
    """.strip()


# this connects the formatter to the insights run CLI bits
class SimpleHtmlFormatterAdapter(FormatterAdapter):
    """ Displays results in a simple html format. """

    def preprocess(self, broker):
        self.formatter = SimpleHtmlFormat(broker)
        self.formatter.preprocess()

    def postprocess(self, broker):
        self.formatter.postprocess()



"""
SambaConfig - file ``/etc/samba/smb.conf``
==========================================

This parser reads the SaMBa configuration file ``/etc/samba/smb.conf``, which
is in standard .ini format, with a couple of notable features:

* SaMBa ignores spaces at the start of options, which the ConfigParser class
  normally does not.  This spacing is stripped by this parser.
* SaMBa likewise ignores spaces in section heading names.
* SaMBa allows the same section to be defined multiple times, with the
  options therein being merged as if they were one section.
* SaMBa allows options to be declared before the first section marker.
  This parser puts these options in a `global` section.
* SaMBa treats ';' as a comment prefix, similar to '#'.

Sample configuration file::

    # This is the main Samba configuration file. You should read the
    # smb.conf(5) manual page in order to understand the options listed
    #...
    #======================= Global Settings =====================================

    [global]
        workgroup = MYGROUP
        server string = Samba Server Version %v
        max log size = 50

    [homes]
        comment = Home Directories
        browseable = no
        writable = yes
    ;   valid users = %S
    ;   valid users = MYDOMAIN\%S

    [printers]
        comment = All Printers
        path = /var/spool/samba
        browseable = no
        guest ok = no
        writable = no
        printable = yes

    # A publicly accessible directory, but read only, except for people in
    # the "staff" group
    [public]
       comment = Public Stuff
       path = /home/samba
       public = yes
       writable = yes
       printable = no
       write list = +staff

Examples:

    >>> type(conf)
    <class 'insights.parsers.samba.SambaConfig'>
    >>> sorted(conf.sections()) == [u'global', u'homes', u'printers', u'public']
    True
    >>> global_options = conf.items('global')  # get a section as a dictionary
    >>> type(global_options) == type({})
    True
    >>> conf.get('public', 'comment') == u'Public Stuff'  # Accessor for section and option
    True
    >>> conf.getboolean('public', 'writable')  # Type conversion, but no default
    True
    >>> conf.getint('global', 'max log size')  # Same for integer conversion
    50

"""

from .. import add_filter, IniConfigFile, parser
from insights.specs import Specs

add_filter(Specs.samba, ["["])


@parser(Specs.samba)
class SambaConfig(IniConfigFile):
    """
    This parser reads the SaMBa configuration file ``/etc/samba/smb.conf``.
    """

    def parse_content(self, content):
        # smb.conf is special from other ini files in the property that
        # whatever is before the first section (before the first section)
        # belongs to the [global] section. Therefore, the [global] section is
        # appended right at the beginning so that everything that would be
        # parsed as outside section belongs to [global].
        # Python 2.7 RawConfigParser automatically merges multiple instances
        # of the same section.  (And if that ever changes, test_samba.py will
        # catch it.)
        lstripped = ["[global]"] + [line.lstrip() for line in content]

        super(SambaConfig, self).parse_content(lstripped)

        # Create a new instance of the same dict type used by the underlying
        # RawConfigParser.
        new_dict = self.data._dict()
        # Transform the section names so that whitespace around is stripped
        # and they are lowercase. smb.conf is special in the property that
        # section names and option names are case-insensitive and treated
        # like lower-case.
        for old_key, old_section in self.data._sections.items():
            new_key = old_key.strip().lower()
            if new_key not in new_dict:
                new_dict[new_key] = self.data._dict()
            # Merge same-named sections just as samba's `testparm` does.
            new_dict[new_key].update(old_section)
        self.data._sections = new_dict



from __future__ import print_function
from __future__ import absolute_import
# Copyright (C) 2016 Red Hat, All rights reserved.
# AUTHORS: William Temple <wtemple@redhat.com>
#          Brent Baude    <bbaude@redhat.com>
# Augments for POC by:
#          Alex Collins   <alcollin@redhat.com>

import os
import sys
import json

from .constants import InsightsConstants as constants

try:
    import docker
except ImportError:
    print("The docker-py Python libraries do not appear to be installed. Please install the python-docker-py RPM package.")
    sys.exit(constants.sig_kill_bad)
from fnmatch import fnmatch as matches

from . import util
from . import dmsetupWrap
from docker.utils import kwargs_from_env


""" Module for mounting and unmounting containerized applications. """


class MountError(Exception):

    """Generic error mounting a candidate container."""

    def __init__(self, val):
        self.val = val

    def __str__(self):
        return str(self.val)


class SelectionMatchError(MountError):

    """Input identifier matched multiple mount candidates."""

    def __init__(self, i, matches):
        self.val = ('"{0}" matched multiple items. Try one of the following:\n'
                    '{1}'.format(i, '\n'.join(['\t' + m for m in matches])))


class Mount:

    """
    A class which contains backend-independent methods useful for mounting and
    unmounting containers.
    """

    def __init__(self, mountpoint):
        """
        Constructs the Mount class with a mountpoint.
        Optional: mount a running container live (read/write)
        """
        self.mountpoint = mountpoint

    def mount(self, identifier, options=[]):
        raise NotImplementedError('Mount subclass does not implement mount() '
                                  'method.')

    def unmount(self):
        raise NotImplementedError('Mount subclass does not implement unmount()'
                                  ' method.')

    # LVM DeviceMapper Utility Methods
    @staticmethod
    def _activate_thin_device(name, dm_id, size, pool):
        """
        Provisions an LVM device-mapper thin device reflecting,
        DM device id 'dm_id' in the docker pool.
        """
        table = '0 %d thin /dev/mapper/%s %s' % (int(size) // 512, pool, dm_id)
        cmd = ['dmsetup', 'create', name, '--table', table]
        r = util.subp(cmd)
        if r.return_code != 0:
            raise MountError('Failed to create thin device: %s' %
                             r.stderr.decode(sys.getdefaultencoding()))

    @staticmethod
    def remove_thin_device(name, force=False):
        """
        Destroys a thin device via subprocess call.
        """
        cmd = ['dmsetup', 'remove', '--retry', name]
        r = util.subp(cmd)
        if not force:
            if r.return_code != 0:
                raise MountError('Could not remove thin device:\n%s' %
                                 r.stderr.decode(sys.getdefaultencoding()).split("\n")[0])

    @staticmethod
    def _is_device_active(device):
        """
        Checks dmsetup to see if a device is already active
        """
        cmd = ['dmsetup', 'info', device]
        dmsetup_info = util.subp(cmd)
        for dm_line in dmsetup_info.stdout.split("\n"):
            line = dm_line.split(':')
            if ('State' in line[0].strip()) and ('ACTIVE' in line[1].strip()):
                return True
        return False

    @staticmethod
    def _get_fs(thin_pathname):
        """
        Returns the file system type (xfs, ext4) of a given device
        """
        cmd = ['lsblk', '-o', 'FSTYPE', '-n', thin_pathname]
        fs_return = util.subp(cmd)
        return fs_return.stdout.strip()

    @staticmethod
    def mount_path(source, target, bind=False):
        """
        Subprocess call to mount dev at path.
        """
        cmd = ['mount']
        if bind:
            cmd.append('--bind')
        cmd.append(source)
        cmd.append(target)
        r = util.subp(cmd)
        if r.return_code != 0:
            raise MountError('Could not mount docker container:\n' +
                             ' '.join(cmd) + '\n%s' %
                             r.stderr.decode(sys.getdefaultencoding()))

    @staticmethod
    def get_dev_at_mountpoint(mntpoint):
        """
        Retrieves the device mounted at mntpoint, or raises
        MountError if none.
        """
        results = util.subp(['findmnt', '-o', 'SOURCE', mntpoint])
        if results.return_code != 0:
            raise MountError('No device mounted at %s' % mntpoint)

        stdout = results.stdout.decode(sys.getdefaultencoding())
        return stdout.replace('SOURCE\n', '').strip().split('\n')[-1]

    @staticmethod
    def unmount_path(path, force=False):
        """
        Unmounts the directory specified by path.
        """
        r = util.subp(['umount', path])
        if not force:
            if r.return_code != 0:
                raise ValueError(r.stderr)


class DockerMount(Mount):

    """
    A class which can be used to mount and unmount docker containers and
    images on a filesystem location.

    mnt_mkdir = Create temporary directories based on the cid at mountpoint
                for mounting containers
    """

    def __init__(self, mountpoint, mnt_mkdir=False):
        Mount.__init__(self, mountpoint)
        self.client = docker.Client(**kwargs_from_env())
        self.mnt_mkdir = mnt_mkdir
        self.tmp_image = None

    def _create_temp_container(self, iid):
        """
        Create a temporary container from a given iid.

        Temporary containers are marked with a sentinel environment
        variable so that they can be cleaned on unmount.
        """
        try:
            return self.client.create_container(
                image=iid, command='/bin/true',
                environment=['_ATOMIC_TEMP_CONTAINER'],
                detach=True, network_disabled=True)['Id']
        except docker.errors.APIError as ex:
            raise MountError('Error creating temporary container:\n%s' % str(ex))

    def _clone(self, cid):
        """
        Create a temporary image snapshot from a given cid.

        Temporary image snapshots are marked with a sentinel label
        so that they can be cleaned on unmount.
        """
        try:
            iid = self.client.commit(
                container=cid,
                conf={
                    'Labels': {
                        'io.projectatomic.Temporary': 'true'
                    }
                }
            )['Id']
        except docker.errors.APIError as ex:
            raise MountError(str(ex))
        self.tmp_image = iid
        return self._create_temp_container(iid)

    def _is_container_running(self, cid):
        cinfo = self.client.inspect_container(cid)
        return cinfo['State']['Running']

    def _identifier_as_cid(self, identifier):
        """
        Returns a container uuid for identifier.

        If identifier is an image UUID or image tag, create a temporary
        container and return its uuid.
        """
        def __cname_matches(container, identifier):
            return any([n for n in (container['Names'] or [])
                        if matches(n, '/' + identifier)])

        # Determine if identifier is a container
        containers = [c['Id'] for c in self.client.containers(all=True)
                      if (__cname_matches(c, identifier) or
                          matches(c['Id'], identifier + '*'))]

        if len(containers) > 1:
            raise SelectionMatchError(identifier, containers)
        elif len(containers) == 1:
            c = containers[0]
            return self._clone(c)

        # Determine if identifier is an image UUID
        images = [i for i in set(self.client.images(all=True, quiet=True))
                  if i.startswith(identifier)]

        if len(images) > 1:
            raise SelectionMatchError(identifier, images)
        elif len(images) == 1:
            return self._create_temp_container(images[0])

        # Match image tag.
        images = util.image_by_name(identifier)
        if len(images) > 1:
            tags = [t for i in images for t in i['RepoTags']]
            raise SelectionMatchError(identifier, tags)
        elif len(images) == 1:
            return self._create_temp_container(images[0]['Id'].replace("sha256:", ""))

        raise MountError('{} did not match any image or container.'
                         ''.format(identifier))

    @staticmethod
    def _no_gd_api_dm(cid):
        # TODO: Deprecated
        desc_file = os.path.join('/var/lib/docker/devicemapper/metadata', cid)
        desc = json.loads(open(desc_file).read())
        return desc['device_id'], desc['size']

    @staticmethod
    def _no_gd_api_overlay(cid):
        # TODO: Deprecated
        prefix = os.path.join('/var/lib/docker/overlay/', cid)
        ld_metafile = open(os.path.join(prefix, 'lower-id'))
        ld_loc = os.path.join('/var/lib/docker/overlay/', ld_metafile.read())
        return (os.path.join(ld_loc, 'root'), os.path.join(prefix, 'upper'),
                os.path.join(prefix, 'work'))

    def mount(self, identifier):
        """
        Mounts a container or image referred to by identifier to
        the host filesystem.
        """

        driver = self.client.info()['Driver']
        driver_mount_fn = getattr(self, "_mount_" + driver,
                                  self._unsupported_backend)
        cid = driver_mount_fn(identifier)

        # Return mount path so it can be later unmounted by path
        return self.mountpoint, cid

    def _unsupported_backend(self, identifier=''):
        raise MountError('Insights cannot be used with the {} docker '
                         'storage backend.'
                         ''.format(self.client.info()['Driver']))

    def _mount_devicemapper(self, identifier):
        """
        Devicemapper mount backend.
        """

        info = self.client.info()

        # cid is the contaienr_id of the temp container
        cid = self._identifier_as_cid(identifier)

        cinfo = self.client.inspect_container(cid)

        dm_dev_name, dm_dev_id, dm_dev_size = '', '', ''
        dm_pool = info['DriverStatus'][0][1]

        try:
            dm_dev_name = cinfo['GraphDriver']['Data']['DeviceName']
            dm_dev_id = cinfo['GraphDriver']['Data']['DeviceId']
            dm_dev_size = cinfo['GraphDriver']['Data']['DeviceSize']
        except:
            # TODO: deprecated when GraphDriver patch makes it upstream
            dm_dev_id, dm_dev_size = DockerMount._no_gd_api_dm(cid)
            dm_dev_name = dm_pool.replace('pool', cid)

        # grab list of devces
        dmsetupLs = dmsetupWrap.getDmsetupLs()
        if dmsetupLs == -1:
            raise MountError('Error: dmsetup returned non zero error ')

        # ENSURE device exists!
        if dm_dev_name not in dmsetupLs:
            # IF device doesn't exist yet we create it!
            Mount._activate_thin_device(dm_dev_name, dm_dev_id, dm_dev_size,
                                        dm_pool)

        # check that device is shown in /dev/mapper, if not we can use the
        # major minor numbers in /dev/block
        mapperDir = os.path.join('/dev/mapper', dm_dev_name)
        if os.path.exists(mapperDir):
            dm_dev_path = mapperDir
        else:
            # get new dmsetupLs after device has been created!
            dmsetupLs = dmsetupWrap.getDmsetupLs()
            # test if device exists in dmsetupls, if so, get its majorminor found in /dev/block
            majorMinor = dmsetupWrap.getMajorMinor(dm_dev_name, dmsetupLs)
            blockDir = os.path.join('/dev/block', majorMinor)

            # FIXME, coudl be due to Virtual box, but occasionally the block device
            # will not be created by the time we check it exists below, so we
            # can wait a half a second to let it be created up
            import time
            time.sleep(0.1)

            if os.path.exists(blockDir):
                dm_dev_path = blockDir
            else:
                raise MountError('Error: Block device found in dmsetup ls '
                                 'but not in /dev/mapper/ or /dev/block')

        options = ['ro', 'nosuid', 'nodev']
        # XFS should get nouuid
        fstype = Mount._get_fs(dm_dev_path).decode(sys.getdefaultencoding())
        if fstype.upper() == 'XFS' and 'nouuid' not in options:
            if 'nouuid' not in options:
                options.append('nouuid')
        try:
            Mount.mount_path(dm_dev_path, self.mountpoint)
        except MountError as de:
            self._cleanup_container(cinfo)
            Mount.remove_thin_device(dm_dev_name)
            raise de

        # return the temp container ID so we can unmount later
        return cid

    def _mount_overlay(self, identifier):
        """
        OverlayFS mount backend.
        """

        cid = self._identifier_as_cid(identifier)
        cinfo = self.client.inspect_container(cid)

        ld, ud, wd = '', '', ''
        try:
            ld = cinfo['GraphDriver']['Data']['lowerDir']
            ud = cinfo['GraphDriver']['Data']['upperDir']
            wd = cinfo['GraphDriver']['Data']['workDir']
        except:
            ld, ud, wd = DockerMount._no_gd_api_overlay(cid)

        options = ['ro', 'lowerdir=' + ld, 'upperdir=' + ud, 'workdir=' + wd]
        optstring = ','.join(options)
        cmd = ['mount', '-t', 'overlay', '-o', optstring, 'overlay',
               self.mountpoint]
        status = util.subp(cmd)
        if status.return_code != 0:
            self._cleanup_container(cinfo)
            raise MountError('Failed to mount OverlayFS device.\n%s' %
                             status.stderr.decode(sys.getdefaultencoding()))

        return cid

    def _cleanup_container(self, cinfo):
        """
        Remove a container and clean up its image if necessary.
        """
        # I'm not a fan of doing this again here.
        env = cinfo['Config']['Env']
        if (env and '_ATOMIC_TEMP_CONTAINER' not in env) or not env:
            return

        iid = cinfo['Image']
        self.client.remove_container(cinfo['Id'])
        try:
            labels = self.client.inspect_image(iid)['Config']['Labels']
        except TypeError:
            labels = {}
        if labels and 'io.projectatomic.Temporary' in labels:
            if labels['io.projectatomic.Temporary'] == 'true':
                self.client.remove_image(iid)

    def _clean_tmp_image(self):
        # If a temporary image is created with commit,
        # clean up that too
        if self.tmp_image is not None:
            self.client.remove_image(self.tmp_image, noprune=True)

    def unmount(self, cid):
        """
        Unmounts and cleans-up after a previous mount().
        """
        driver = self.client.info()['Driver']
        driver_unmount_fn = getattr(self, "_unmount_" + driver,
                                    self._unsupported_backend)
        driver_unmount_fn(cid)

    def _unmount_devicemapper(self, cid):
        """
        Devicemapper unmount backend.
        """
        mountpoint = self.mountpoint
        Mount.unmount_path(mountpoint)

        cinfo = self.client.inspect_container(cid)
        dev_name = cinfo['GraphDriver']['Data']['DeviceName']

        Mount.remove_thin_device(dev_name)
        self._cleanup_container(cinfo)

    def _unmount_overlay(self, cid):
        """
        OverlayFS unmount backend.
        """
        mountpoint = self.mountpoint
        Mount.unmount_path(mountpoint)
        self._cleanup_container(self.client.inspect_container(cid))

    def _clean_temp_container_by_path(self, path):
        short_cid = os.path.basename(path)
        self.client.remove_container(short_cid)
        self._clean_tmp_image()



from insights.parsers.uname import Uname
from insights.parsers.redhat_release import RedhatRelease
from insights.combiners.redhat_release import redhat_release, RedHatRelease
from insights.combiners import redhat_release as rr
from insights.tests import context_wrap
from insights.parsers import SkipComponent
import pytest
import doctest

UNAME = "Linux localhost.localdomain 3.10.0-327.rt56.204.el7.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
BAD_UNAME = "Linux localhost.localdomain 2.6.24.7-101.el5rt.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"

REDHAT_RELEASE = """
Red Hat Enterprise Linux Server release 7.2 (Maipo)
""".strip()

FEDORA = """
Fedora release 23 (Twenty Three)
""".strip()


def test_uname():
    un = Uname(context_wrap(UNAME))
    expected = (7, 2)
    result = redhat_release(None, un)
    assert result.major == expected[0]
    assert result.minor == expected[1]


def test_redhat_release():
    rel = RedhatRelease(context_wrap(REDHAT_RELEASE))
    expected = (7, 2)
    result = redhat_release(rel, None)
    assert result.major == expected[0]
    assert result.minor == expected[1]


def test_both():
    un = Uname(context_wrap(UNAME))
    rel = RedhatRelease(context_wrap(REDHAT_RELEASE))
    expected = (7, 2)
    result = redhat_release(rel, un)
    assert result.major == expected[0]
    assert result.minor == expected[1]


def test_RedHatRelease_uname():
    un = Uname(context_wrap(UNAME))
    expected = (7, 2)
    result = RedHatRelease(un, None)
    assert result.major == expected[0]
    assert result.minor == expected[1]
    assert result.rhel == result.rhel7 == '7.2'
    assert result.rhel6 is None


def test_RedHatRelease_redhat_release():
    rel = RedhatRelease(context_wrap(REDHAT_RELEASE))
    expected = (7, 2)
    result = RedHatRelease(None, rel)
    assert result.major == expected[0]
    assert result.minor == expected[1]
    assert result.rhel == result.rhel7 == '7.2'
    assert result.rhel8 is None


def test_RedHatRelease_both():
    un = Uname(context_wrap(UNAME))
    rel = RedhatRelease(context_wrap(REDHAT_RELEASE))
    expected = (7, 2)
    result = RedHatRelease(un, rel)
    assert result.major == expected[0]
    assert result.minor == expected[1]
    assert result.rhel == result.rhel7 == '7.2'
    assert result.rhel6 is None
    assert result.rhel8 is None


def test_raise():
    un = Uname(context_wrap(BAD_UNAME))
    with pytest.raises(SkipComponent):
        redhat_release(None, un)

    with pytest.raises(SkipComponent):
        RedHatRelease(un, None)

    with pytest.raises(SkipComponent):
        RedHatRelease(None, None)


def test_doc_examples():
    env = {
            'rh_release': redhat_release(None, Uname(context_wrap(UNAME))),
            'rh_rel': RedHatRelease(Uname(context_wrap(UNAME)), None),
          }
    failed, total = doctest.testmod(rr, globs=env)
    assert failed == 0



"""
IsRhel6, IsRhel7 and IsRhel8
===============================

An ``IsRhel*`` component is valid if the
:py:class:`insights.combiners.redhat_release.RedHatRelease` combiner indicates
the major RHEL version represented by the component. Otherwise, it raises a
:py:class:`insights.core.dr.SkipComponent` to prevent dependent components from
executing.

In particular, an ``IsRhel*`` component can be added as a dependency of a
parser to limit it to a given version.
"""

from insights.core.plugins import component
from insights.combiners.redhat_release import RedHatRelease
from insights.core.dr import SkipComponent


@component(RedHatRelease)
class IsRhel6(object):
    """
    This component uses ``RedHatRelease`` combiner
    to determine RHEL version. It checks if RHEL6, if not
    RHEL6 it raises ``SkipComponent``.

    Raises:
        SkipComponent: When RHEL version is not RHEL6.
    """
    def __init__(self, rhel):
        if rhel.major != 6:
            raise SkipComponent('Not RHEL6')


@component(RedHatRelease)
class IsRhel7(object):
    """
    This component uses ``RedHatRelease`` combiner
    to determine RHEL version. It checks if RHEL7, if not \
    RHEL7 it raises ``SkipComponent``.

    Raises:
        SkipComponent: When RHEL version is not RHEL7.
    """
    def __init__(self, rhel):
        if rhel.major != 7:
            raise SkipComponent('Not RHEL7')


@component(RedHatRelease)
class IsRhel8(object):
    """
    This component uses ``RedhatRelease`` combiner
    to determine RHEL version. It checks if RHEL8, if not
    RHEL8 it raises ``SkipComponent``.

    Raises:
        SkipComponent: When RHEL version is not RHEL8.
    """
    def __init__(self, rhel):
        if rhel.major != 8:
            raise SkipComponent('Not RHEL8')



import pytest

from insights.tests import context_wrap
from insights.parsers import ParseException
from insights.parsers.mongod_conf import MongodbConf


NORMAL_CONF = """
# mongodb.conf - generated from Puppet

#where to log
logpath=/var/log/mongodb/mongodb.log
logappend=true
# Set this option to configure the mongod or mongos process to bind to and
# listen for connections from applications on this address.
# You may concatenate a list of comma separated values to bind mongod to multiple IP addresses.
bind_ip = 127.0.0.1
# fork and run in background
fork=true
dbpath=/var/lib/mongodb
# location of pidfile
pidfilepath=/var/run/mongodb/mongodb.pid
# Enables journaling
journal = true
# Turn on/off security.  Off is currently the default
noauth=true
abc=
""".strip()


NORMAL_CONF_V1 = """
=/var/log/mongodb/mongodb.log
logappend=true # noauth=true
""".strip()


YAML_CONF = """
# mongod.conf

# for documentation of all options, see:
#   http://docs.mongodb.org/manual/reference/configuration-options/

# where to write logging data.
systemLog:
  destination: file
  logAppend: true
  path: /var/log/mongodb/mongod.log

# Where and how to store data.
storage:
  dbPath: /var/lib/mongo
  journal:
    enabled: true
#  engine:
#  mmapv1:
#  wiredTiger:

# how the process runs
processManagement:
  fork: true  # fork and run in background
  pidFilePath: /var/run/mongodb/mongod.pid  # location of pidfile

# network interfaces
net:
  port: 27017
  #bindIp: 127.0.0.1  # Listen to local interface only, comment to listen on all interfaces.
  #bindIp: 127.0.0.1  # Listen to local interface only, comment to listen on all interfaces.


#security:

#operationProfiling:

#replication:

#sharding:

## Enterprise-Only Options

#auditLog:

#snmp:
""".strip()

YAML_CONF_UNPARSABLE = """
systemLog:
  destination: file
  logAppend: true
  port=27017

""".strip()


def test_mongodb_conf():

    result = MongodbConf(context_wrap(YAML_CONF))
    assert result.get("security") is None
    assert result.get("processManagement") == {
            'fork': True,
            'pidFilePath': '/var/run/mongodb/mongod.pid'}
    assert result.is_yaml is True
    assert result.port == 27017
    assert result.bindip is None
    assert result.dbpath == '/var/lib/mongo'
    assert result.fork is True
    assert result.pidfilepath == '/var/run/mongodb/mongod.pid'
    assert result.syslog == 'file'
    assert result.logpath == '/var/log/mongodb/mongod.log'

    result = MongodbConf(context_wrap(NORMAL_CONF))
    assert result.is_yaml is False
    assert result.port is None
    assert result.bindip == '127.0.0.1'
    assert result.dbpath == '/var/lib/mongodb'
    assert result.fork == 'true'
    assert result.pidfilepath == '/var/run/mongodb/mongodb.pid'
    assert result.syslog is None
    assert result.logpath == '/var/log/mongodb/mongodb.log'
    assert result.get("abc") == ''
    assert result.get("def") is None

    result = MongodbConf(context_wrap(NORMAL_CONF_V1))
    assert result.is_yaml is False
    assert len(result.data) == 2
    assert result.get("logappend") == 'true'

    with pytest.raises(ParseException) as e:
        MongodbConf(context_wrap(YAML_CONF_UNPARSABLE))
    assert "mongod conf parse failed:" in str(e.value)

    # add test for empty mongod.conf for "ZeroDivisionError: float division by zero"
    with pytest.raises(ParseException) as e:
        MongodbConf(context_wrap("#this is empty"))
    assert "mongod.conf is empty or all lines are comments" in str(e.value)



#!/usr/bin/env python
"""
Command Parser module
---------------------
Implements the ``insights`` command line.  Each function is the first
argument followed by the function specific arguments.  See USAGE text
below.
"""
from __future__ import print_function

import argparse
import sys

USAGE = """insights <command> [<args>]
Available commands:
  cat         Execute a spec and show the output
  collect     Collect all specs against the client and create an Insights archive.
  inspect     Execute component and shell out to ipython for evaluation.
  info        View info and docs for Insights Core components.
  run         Run insights-core against host or an archive.
"""


class InsightsCli(object):
    """
    Class to implement the cli module.
    Each command is called as a method of this class and all
    arg parsing is performed in the separate module that
    actually implements the command.  the args "insights command"
    are not passed to the submodule.
    """

    def __init__(self):
        parser = argparse.ArgumentParser(
            description="Insights Core command line execution",
            usage=USAGE)
        parser.add_argument('command', help='Insights Core command to run')
        args = parser.parse_args(sys.argv[1:2])
        if not hasattr(self, args.command):
            print('Unrecognized command')
            parser.print_help()
            sys.exit(1)
        # remove the sub-command arg from sys.argv
        sys.argv.pop(1)
        # Use dispatch pattern to execute command
        getattr(self, args.command)()

    def cat(self):
        from .tools.cat import main as cat_main
        cat_main()

    def collect(self):
        from .collect import main as collect_main
        collect_main()

    def info(self):
        from .tools.query import main as query_main
        query_main()

    def inspect(self):
        from .tools.insights_inspect import main as inspect_main
        inspect_main()

    def run(self):
        from insights import run
        if "" not in sys.path:
            sys.path.insert(0, "")
        run(print_summary=True)


def fix_arg_dashes():

    en_dash = '\u2013'
    em_dash = '\u2014'

    # replace unicode (en dash and em dash) dashes from argument definitions that may have been copy
    # and pasted from another source
    i = 1
    for a in sys.argv[1:]:
        first = list(a)
        first[0] = first[0].replace(em_dash, "--").replace(en_dash, "-")
        sys.argv[i] = "".join(first)
        i += 1


def main():
    fix_arg_dashes()
    try:
        InsightsCli()
    except SystemExit:
        raise
    except BaseException as ex:
        print(ex)


if __name__ == "__main__":
    main()



"""
Sap
===
This combiner combines the result of
insights.parsers.saphostctrl.SAPHostCtrlInstances` and
`:class:`insights.parsers.lssap.Lssap` to get the available SAP instances.
Prefer the ``SAPHostCtrlInstances`` to ``Lssap``.

"""

from collections import namedtuple
from insights import SkipComponent
from insights.core.plugins import combiner
from insights.combiners.hostname import hostname
from insights.parsers.lssap import Lssap
from insights.parsers.saphostctrl import SAPHostCtrlInstances


SAPInstances = namedtuple("SAPInstances",
        field_names=["name", "hostname", "sid", "type", "number", "fqdn", "version"])
"""namedtuple: Type for storing the SAP instance."""


@combiner(hostname, [SAPHostCtrlInstances, Lssap])
class Sap(dict):
    """
    Combiner for combining the result of :class:`insights.parsers.lssap.Lssap`
    generated by command ``lssap`` and
    `insights.parsers.saphostctrl.SAPHostCtrlInstances` generated by command
    ``saphostctrl``.

    Prefer ``SAPHostCtrlInstances`` to ``Lssap``.

    Examples:
        >>> type(saps)
        <class 'insights.combiners.sap.Sap'>
        >>> 'D16' in saps
        True
        >>> saps['D16'].number
        '16'
        >>> saps.sid('HDB16')
        'HA2'
        >>> saps.hostname('HDB16')
        'lu0417'
        >>> len(saps.business_instances)
        3
        >>> saps.is_hana
        True
        >>> saps.is_netweaver
        True
        >>> saps.is_ascs
        False

    Attributes:
        all_instances (list): List of all the SAP instances listed by the command.
        function_instances (list): List of functional SAP instances
                                   E.g. Diagnostics Agents SMDA97/SMDA98
        business_instances (list): List of business SAP instances
                                   E.g. HANA, NetWeaver, ASCS, or others
        local_instances (list): List of all SAP instances running on this host
    """
    FUNC_INSTS = ('SMDA',)
    """ tuple: Tuple of the prefix string of the functional SAP instances"""

    def __init__(self, hostname, insts, lssap):
        hn = hostname.hostname
        self.data = {}
        self.local_instances = []
        self.business_instances = []
        self.function_instances = []
        self.all_instances = []
        self._types = set()
        if insts:
            for inst in insts.data:
                k = inst['InstanceName']
                self.all_instances.append(k)
                self._types.add(inst['InstanceType'])
                self.local_instances.append(k) if hn == inst['Hostname'] else None
                self.data[k] = SAPInstances(k,
                                            inst['Hostname'],
                                            inst['SID'],
                                            inst['InstanceType'],
                                            inst['SystemNumber'],
                                            inst['FullQualifiedHostname'],
                                            inst['SapVersionInfo'])
        elif lssap:
            for inst in lssap.data:
                k = inst['Instance']
                t = k.rstrip('1234567890')
                self.all_instances.append(k)
                self._types.add(t)
                self.local_instances.append(k) if hn == inst['SAPLOCALHOST'] else None
                self.data[k] = SAPInstances(k,
                                            inst['SAPLOCALHOST'],
                                            inst['SID'],
                                            t,
                                            inst['Nr'],
                                            None,
                                            inst['Version'])
        if not self.data:
            raise SkipComponent('No SAP instance.')

        self.update(self.data)

        for i in self.all_instances:
            (self.function_instances if i.startswith(self.FUNC_INSTS) else self.business_instances).append(i)

    def version(self, instance):
        """str: Returns the version of the ``instance``."""
        return self.data[instance].version if instance in self.data else None

    def sid(self, instance):
        """str: Returns the sid of the ``instance``."""
        return self.data[instance].sid if instance in self.data else None

    def type(self, instance):
        """str: Returns the type code of the ``instance``."""
        return self.data[instance].type if instance in self.data else None

    def hostname(self, instance):
        """str: Returns the hostname of the ``instance``."""
        return self.data[instance].hostname if instance in self.data else None

    def number(self, instance):
        """str: Returns the systeme number of the ``instance``."""
        return self.data[instance].number if instance in self.data else None

    @property
    def is_netweaver(self):
        """bool: Is any SAP NetWeaver instance detected?"""
        return 'D' in self._types

    @property
    def is_hana(self):
        """bool: Is any SAP HANA instance detected?"""
        return 'HDB' in self._types

    @property
    def is_ascs(self):
        """bool: Is any SAP System Central Services instance detected?"""
        return 'ASCS' in self._types



"""
RsyslogConf - file ``/etc/rsyslog.conf``
========================================

The rsyslog configuration files can include statements with two different
line based formats along with snippets of 'RainerScript' that can span
multiple lines.

See http://www.rsyslog.com/doc/master/configuration/basic_structure.html#statement-types

Due to high parsing complexity, this parser presents a simple line-based
view of the file that meets the needs of the current rules.

Example:
    >>> content = '''
    ... :fromhost-ip, regex, "10.0.0.[0-9]" /tmp/my_syslog.log
    ... $ModLoad imtcp
    ... $InputTCPServerRun 10514"
    ... '''.strip()
    >>> from insights.tests import context_wrap
    >>> rsl = RsyslogConf(context_wrap(content))
    >>> len(rsl)
    3
    >>> len(list(rsl))
    3
    >>> any('imtcp' in n for n in rsl)
    True
"""
from .. import Parser, parser, get_active_lines

import re
from insights.specs import Specs


@parser(Specs.rsyslog_conf)
class RsyslogConf(Parser):
    """
    Parses `/etc/rsyslog.conf` info simple lines.

    Skips lines that begin with hash ("#") or are only whitespace.

    Attributes:
        data (list): List of lines in the file that don't start
            with '#' and aren't whitespace.
        config_items(dict): Configuration items opportunistically found in the
            configuration file, with their values as given.
    """

    def parse_content(self, content):
        self.data = get_active_lines(content)

        self.config_items = {}
        # Config items are e.g. "$Word value #optional comment"
        config_re = re.compile(r'^\s*\$(?P<name>\S+)\s+(?P<value>.*?)(?:\s+#.*)?$')
        for line in self.data:
            lstrip = line.strip()
            match = config_re.match(lstrip)
            if match:
                self.config_items[match.group('name')] = match.group('value')

    def config_val(self, item, default=None):
        """
        Return the given configuration item, or the default if not defined.

        Parameters:
            item(str): The configuration item name
            default: The default if the item is not found (defaults to None)

        Returns:
            The related value in the `config_items` dictionary.
        """
        return self.config_items.get(item, default)

    def __len__(self):
        return len(self.data)

    def __iter__(self):
        for d in self.data:
            yield d



from insights import combiner
from insights.combiners.hostname import hostname
from insights.core.context import create_product
from insights.parsers.metadata import MetadataJson
from insights.specs import Specs


@combiner(MetadataJson, [hostname, Specs.machine_id])
def multinode_product(md, hn, machine_id):
    hn = hn.fqdn if hn else machine_id.content[0].rstrip()
    return create_product(md.data, hn)


@combiner(multinode_product)
def docker(product):
    if product and product.name == "docker":
        return product


@combiner(multinode_product)
def OSP(product):
    if product and product.name == "osp":
        return product


@combiner(multinode_product)
def RHEV(product):
    if product and product.name == "rhev":
        return product


@combiner(multinode_product)
def RHEL(product):
    if product and product.name == "rhel":
        return product



from insights.parsers.dmsetup import DmsetupInfo
from insights.tests import context_wrap

DMSETUP_INFO_1 = """
Name               Maj Min Stat Open Targ Event  UUID
VG00-tmp           253   8 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4
VG00-var_tmp       253   4 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxXOT2ZNHpEmJy2g2FpmXfAH1chG4Utm4Q
VG00-home          253   3 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxCqXOnbGe2zjhX923dFiIdl1oi7mO9tXp
VG00-var_log       253   9 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxlycWK5qprImfYnbkZLNiFZ5Lc6rJq04Z
VG00-usr           253   2 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxJqQ6DLofdR0uWkTnlpkRnFShO3PgqhCT
VG00-var           253   6 L--w    1    2      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxicvyvt67113nTb8vMlGfgdEjDx0LKT2O
VG00-swap          253   1 L--w    2    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax3Ll2XhOYZkylx1CjOQi7G4yHgrIOsyqG
VG00-root          253   0 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxKpnAKYhrYMYMNMwjegkW965bUgtJFTRY
VG00-var_log_audit 253   5 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxwQ8R0XWJRm86QX3befq1cHRy47Von6ZW
VG_DB-vol01        253  10 L--w    1    2      0 LVM-dgoBx4rat9aLu3sg1k95D7YwUT7YnFddgcZSaU2sjHZMVBlwcNwDjmmGGtfXeIZs
VG00-opt           253   7 L--w    1    4      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxIiCYm5hcvgQdXynPGBfHQLrtE3sqUKT2
""".strip()


def test_dmsetup_info():
    r = DmsetupInfo(context_wrap(DMSETUP_INFO_1))
    assert len(r) == 11
    assert len(r[0]) == 8
    assert r[0]['UUID'] == 'LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4'
    assert r[-1]['Stat'] == 'L--w'
    assert r.names == ['VG00-tmp', 'VG00-var_tmp', 'VG00-home', 'VG00-var_log',
                       'VG00-usr', 'VG00-var', 'VG00-swap', 'VG00-root',
                       'VG00-var_log_audit', 'VG_DB-vol01', 'VG00-opt']
    assert r.names == [dm['Name'] for dm in r]
    assert len(r.by_uuid) == 11



from __future__ import absolute_import
import os
import errno
import shlex
import logging
import six
from subprocess import Popen, PIPE, STDOUT
from tempfile import NamedTemporaryFile
from insights.util import mangle

from .constants import InsightsConstants as constants
from .utilities import determine_hostname

logger = logging.getLogger(__name__)


class InsightsSpec(object):
    '''
    A spec loaded from the uploader.json
    '''
    def __init__(self, config, spec, exclude):
        self.config = config
        # exclusions patterns for this spec
        self.exclude = exclude
        # pattern for spec collection
        self.pattern = spec['pattern'] if spec['pattern'] else None


class InsightsCommand(InsightsSpec):
    '''
    A command spec
    '''
    def __init__(self, config, spec, exclude, mountpoint):
        InsightsSpec.__init__(self, config, spec, exclude)
        self.command = spec['command'].replace(
            '{CONTAINER_MOUNT_POINT}', mountpoint)
        self.archive_path = mangle.mangle_command(self.command)
        self.is_hostname = spec.get('symbolic_name') == 'hostname'
        if not six.PY3:
            self.command = self.command.encode('utf-8', 'ignore')

    def get_output(self):
        '''
        Execute a command through system shell. First checks to see if
        the requested command is executable. Returns (returncode, stdout, 0)
        '''
        if self.is_hostname:
            # short circuit for hostame with internal method
            return determine_hostname()

        # all commands should timeout after a long interval so the client does not hang
        # prepend native nix 'timeout' implementation
        timeout_command = 'timeout -s KILL %s %s' % (
            self.config.cmd_timeout, self.command)

        # ensure consistent locale for collected command output
        cmd_env = {'LC_ALL': 'C',
                   'PATH': '/sbin:/bin:/usr/sbin:/usr/bin',
                   'PYTHONPATH': os.getenv('PYTHONPATH')}
        args = shlex.split(timeout_command)

        # never execute this stuff
        if set.intersection(set(args), constants.command_blacklist):
            raise RuntimeError("Command Blacklist: " + self.command)

        try:
            logger.debug('Executing: %s', args)
            proc0 = Popen(args, shell=False, stdout=PIPE, stderr=STDOUT,
                          bufsize=-1, env=cmd_env, close_fds=True)
        except OSError as err:
            if err.errno == errno.ENOENT:
                logger.debug('Command %s not found', self.command)
                return
            else:
                raise err

        dirty = False

        cmd = "sed -rf " + constants.default_sed_file
        sedcmd = Popen(shlex.split(cmd),
                       stdin=proc0.stdout,
                       stdout=PIPE)
        proc0.stdout.close()
        proc0 = sedcmd

        if self.exclude is not None:
            exclude_file = NamedTemporaryFile()
            exclude_file.write("\n".join(self.exclude).encode('utf-8'))
            exclude_file.flush()
            cmd = "grep -F -v -f %s" % exclude_file.name
            proc1 = Popen(shlex.split(cmd),
                          stdin=proc0.stdout,
                          stdout=PIPE)
            proc0.stdout.close()
            stderr = None
            if self.pattern is None or len(self.pattern) == 0:
                stdout, stderr = proc1.communicate()

            # always log return codes for debug
            logger.debug('Proc1 Status: %s', proc1.returncode)
            logger.debug('Proc1 stderr: %s', stderr)
            proc0 = proc1

            dirty = True

        if self.pattern is not None and len(self.pattern):
            pattern_file = NamedTemporaryFile()
            pattern_file.write("\n".join(self.pattern).encode('utf-8'))
            pattern_file.flush()
            cmd = "grep -F -f %s" % pattern_file.name
            proc2 = Popen(shlex.split(cmd),
                          stdin=proc0.stdout,
                          stdout=PIPE)
            proc0.stdout.close()
            stdout, stderr = proc2.communicate()

            # always log return codes for debug
            logger.debug('Proc2 Status: %s', proc2.returncode)
            logger.debug('Proc2 stderr: %s', stderr)
            proc0 = proc2

            dirty = True

        if not dirty:
            stdout, stderr = proc0.communicate()

        # Required hack while we still pass shell=True to Popen; a Popen
        # call with shell=False for a non-existant binary will raise OSError.
        if proc0.returncode == 126 or proc0.returncode == 127:
            stdout = "Could not find cmd: %s", self.command

        logger.debug("Proc0 Status: %s", proc0.returncode)
        logger.debug("Proc0 stderr: %s", stderr)
        return stdout.decode('utf-8', 'ignore').strip()


class InsightsFile(InsightsSpec):
    '''
    A file spec
    '''
    def __init__(self, spec, exclude, mountpoint):
        InsightsSpec.__init__(self, None, spec, exclude)
        # substitute mountpoint for collection
        self.real_path = os.path.join(mountpoint, spec['file'].lstrip('/'))
        self.archive_path = spec['file']

    def get_output(self):
        '''
        Get file content, selecting only lines we are interested in
        '''
        if not os.path.isfile(self.real_path):
            logger.debug('File %s does not exist', self.real_path)
            return

        cmd = []
        cmd.append('sed')
        cmd.append('-rf')
        cmd.append(constants.default_sed_file)
        cmd.append(self.real_path)
        sedcmd = Popen(cmd,
                       stdout=PIPE)

        if self.exclude is not None:
            exclude_file = NamedTemporaryFile()
            exclude_file.write("\n".join(self.exclude).encode('utf-8'))
            exclude_file.flush()

            cmd = "grep -v -F -f %s" % exclude_file.name
            args = shlex.split(cmd)
            proc = Popen(args, stdin=sedcmd.stdout, stdout=PIPE)
            sedcmd.stdout.close()
            stdin = proc.stdout
            if self.pattern is None:
                output = proc.communicate()[0]
            else:
                sedcmd = proc

        if self.pattern is not None:
            pattern_file = NamedTemporaryFile()
            pattern_file.write("\n".join(self.pattern).encode('utf-8'))
            pattern_file.flush()

            cmd = "grep -F -f %s" % pattern_file.name
            args = shlex.split(cmd)
            proc1 = Popen(args, stdin=sedcmd.stdout, stdout=PIPE)
            sedcmd.stdout.close()

            if self.exclude is not None:
                stdin.close()

            output = proc1.communicate()[0]

        if self.pattern is None and self.exclude is None:
            output = sedcmd.communicate()[0]

        return output.decode('utf-8', 'ignore').strip()



"""
CephVersion - command ``/usr/bin/ceph -v``
==========================================

This module provides plugins access to the Ceph version information gathered from
the ``ceph -v`` command. This module parses the community version to the Red Hat
release version.

The Red Hat Ceph Storage releases and corresponding Ceph package releases are
documented in https://access.redhat.com/solutions/2045583

Typical output of the ``ceph -v`` command is::

    ceph version 0.94.9-9.el7cp (b83334e01379f267fb2f9ce729d74a0a8fa1e92c)

Note:
    This module can only be used for Ceph.

Example:
    >>> ceph_ver = shared[CephVersion]
    >>> ceph_ver.version
    '1.3.3'
    >>> ceph_ver.major
    '1.3'
    >>> ceph_ver.minor
    '3'
"""

from .. import parser, CommandParser
import re
from insights.specs import Specs

community_to_release_map = {
    "0.94.1-15": {'version': "1.3.0", 'major': '1.3', 'minor': '0', 'downstream_release': 'NA'},
    "0.94.3-3": {'version': "1.3.1", 'major': '1.3', 'minor': '1', 'downstream_release': 'NA'},
    "0.94.5-9": {'version': "1.3.2", 'major': '1.3', 'minor': '2', 'downstream_release': 'NA'},
    "0.94.9-3": {'version': "1.3.3", 'major': '1.3', 'minor': '3', 'downstream_release': 'NA'},
    "0.94.9-9": {'version': "1.3.3", 'major': '1.3', 'minor': '3', 'downstream_release': 'async 2'},
    "0.94.10-2": {'version': "1.3.4", 'major': '1.3', 'minor': '4', 'downstream_release': 'NA'},
    "10.2.2-38": {'version': "2.0", 'major': '2', 'minor': '0', 'downstream_release': '0'},
    "10.2.3-13": {'version': "2.1", 'major': '2', 'minor': '1', 'downstream_release': '0'},
    "10.2.5": {'version': "2.2", 'major': '2', 'minor': '2', 'downstream_release': '0'},
    "10.2.5-37": {'version': "2.2", 'major': '2', 'minor': '2', 'downstream_release': '0'},
    "10.2.7-27": {'version': "2.3", 'major': '2', 'minor': '3', 'downstream_release': '0'},
    "10.2.7-28": {'version': "2.3", 'major': '2', 'minor': '3', 'downstream_release': 'async'},
    "10.2.7-48": {'version': "2.4", 'major': '2', 'minor': '4', 'downstream_release': 'async'},
    "10.2.10-16": {'version': "2.5", 'major': '2', 'minor': '5', 'downstream_release': '0'},
    "10.2.10-28": {'version': "2.5.1", 'major': '2', 'minor': '5', 'downstream_release': '1'},
    "10.2.10-40": {'version': "2.5.2", 'major': '2', 'minor': '5', 'downstream_release': '2'},
    "10.2.10-43": {'version': "2.5.3", 'major': '2', 'minor': '5', 'downstream_release': '3'},
    "10.2.10-49": {'version': "2.5.4", 'major': '2', 'minor': '5', 'downstream_release': '4'},
    "12.2.4-6": {'version': "3.0.2", 'major': '3', 'minor': '0', 'downstream_release': '2'},
    "12.2.4-10": {'version': "3.0.3", 'major': '3', 'minor': '0', 'downstream_release': '3'},
    "12.2.4-30": {'version': "3.0.4", 'major': '3', 'minor': '0', 'downstream_release': '4'},
    "12.2.4-42": {'version': "3.0.5", 'major': '3', 'minor': '0', 'downstream_release': '5'},
    "12.2.5-42": {'version': "3.1", 'major': '3', 'minor': '1', 'downstream_release': '0'},
    "12.2.5-59": {'version': "3.1.1", 'major': '3', 'minor': '1', 'downstream_release': '1'},
    "12.2.8-52": {'version': "3.2", 'major': '3', 'minor': '2', 'downstream_release': '0'},
    "12.2.8-89": {'version': "3.2.1", 'major': '3', 'minor': '2', 'downstream_release': '1'},
    "12.2.8-128": {'version': "3.2.2", 'major': '3', 'minor': '2', 'downstream_release': '2'},
}


class CephVersionError(Exception):
    """
    Exception subclass for errors related to the content data and the
    CephVersion class.

    This exception should not be caught by rules plugins unless it is necessary
    for the plugin to return a particular answer when a problem occurs with
    ceph version data.  If a plugin catches this exception it must reraise it so that
    the engine has the opportunity to handle it/log it as necessary.
    """

    def __init__(self, message, errors):
        """Class constructor"""

        super(CephVersionError, self).__init__(message)
        self.errors = errors
        self.message = message


@parser(Specs.ceph_v)
class CephVersion(CommandParser):
    """ Class for parsing the content of ``ceph_version``."""

    def parse_content(self, content):
        # Parse Ceph Version Content and get Release, Major, Minor number
        if not content:
            raise CephVersionError("Empty Ceph Version Line", content)

        ceph_version_line = content[-1]
        # re search pattern
        pattern_community = r'((\d{1,2})\.(\d{1,2})\.((\d{1,2})|x))((\-(\d{1,2}))?)'
        community_version_mo = re.search(pattern_community, str(ceph_version_line), 0)
        if not community_version_mo:
            raise CephVersionError("Wrong Format Ceph Version", content)

        community_version = community_version_mo.group(0)
        release_data = community_to_release_map.get(community_version, None)
        if not release_data:
            raise CephVersionError("No Mapping Release Version. Ceph Release Number is Null", content)

        self.version = release_data['version']
        self.major = release_data['major']
        self.minor = release_data['minor']
        self.downstream_release = release_data['downstream_release']
        self.upstream_version = {
            "release": int(community_version_mo.group(2)),
            "major": int(community_version_mo.group(3)),
            "minor": int(community_version_mo.group(4))
        }



"""
FcoeadmI - command ``fcoeadm -i``
=================================

Module for parsing the output of command ``fcoeadm -i``. The bulk of the
content is split on the colon and keys are kept as is. Lines beginning
with 'Description', 'Revision', 'Manufacturer', 'Serial Number', 'Driver'
,'Number of Ports' are kept in a dictionary keyed under each of these names.
Lines beginning with 'Symbolic Name', 'OS Device Name', 'Node Name', 'Port Name',
'FabricName', 'Speed', 'Supported Speed', 'MaxFrameSize', 'FC-ID (Port ID)',
'State' are kept in a sub-dictionary keyed under each these names. All the
sub-dictionaries are kept in a list keyed in 'Interfaces'.
"""

from insights import parser, CommandParser
from insights.specs import Specs
from insights.parsers import ParseException, SkipException


@parser(Specs.fcoeadm_i)
class FcoeadmI(CommandParser, dict):
    """
    Class for parsing ``fcoeadm -i`` command output.

    Typical output of command ``fcoeadm -i`` looks like::

        Description:      NetXtreme II BCM57810 10 Gigabit Ethernet
        Revision:         10
        Manufacturer:     Broadcom Corporation
        Serial Number:    2C44FD8F4418
        Driver:           bnx2x 1.712.30-0
        Number of Ports:  1

            Symbolic Name:     bnx2fc (QLogic BCM57810) v2.9.6 over eth8.0-fcoe
            OS Device Name:    host6
            Node Name:         0x50060B0000C26237
            Port Name:         0x50060B0000C26236
            FabricName:        0x0000000000000000
            Speed:             Unknown
            Supported Speed:   1 Gbit, 10 Gbit
            MaxFrameSize:      2048
            FC-ID (Port ID):   0xFFFFFFFF
            State:             Online

            Symbolic Name:     bnx2fc (QLogic BCM57810) v2.9.6 over eth6.0-fcoe
            OS Device Name:    host7
            Node Name:         0x50060B0000C26235
            Port Name:         0x50060B0000C26234
            FabricName:        0x0000000000000000
            Speed:             Unknown
            Supported Speed:   1 Gbit, 10 Gbit
            MaxFrameSize:      2048
            FC-ID (Port ID):   0xFFFFFFFF
            State:             Offline

    Examples:
        >>> type(fi)
        <class 'insights.parsers.fcoeadm_i.FcoeadmI'>
        >>> fi.fcoe["Description"]
        'NetXtreme II BCM57810 10 Gigabit Ethernet'
        >>> fi["Description"]
        'NetXtreme II BCM57810 10 Gigabit Ethernet'
        >>> fi.fcoe["Driver"]
        'bnx2x 1.712.30-0'
        >>> fi["Driver"]
        'bnx2x 1.712.30-0'
        >>> fi.fcoe['Serial Number']
        '2C44FD8F4418'
        >>> fi['Serial Number']
        '2C44FD8F4418'
        >>> fi.iface_list
        ['eth8.0-fcoe', 'eth6.0-fcoe']
        >>> fi.nic_list
        ['eth8', 'eth6']
        >>> fi.stat_list
        ['Online', 'Offline']
        >>> fi.get_stat_from_nic('eth6')
        'Offline'
        >>> fi.get_host_from_nic('eth6')
        'host7'

    Attributes:
        fcoe (dict): The result parsed of '/usr/sbin/fcoeadm -i'
        driver_name (str): Driver name
        driver_version (str): Driver version
        iface_list (list) : FCoE interface names
        nic_list (list) : Ethernet ports running FCoE interfaces
        stat_list (list): FCoE interface(s) status

    Raises:
        SkipException: When input content is empty
        ParseException: When input content is not available to parse
    """
    def __init__(self, *args, **kwargs):
        super(FcoeadmI, self).__init__(*args, **kwargs)
        self.update(self.fcoe)

    def parse_content(self, content):
        EMPTY = "Input content is empty"
        BADWD = "No useful data parsed in content: '{0}'".format(content)

        if not content:
            raise SkipException(EMPTY)

        if len(content) < 6 and len(content) > 0:
            raise ParseException(BADWD)

        iface = {}
        iface_section_list = []

        self.iface_list = []
        self.nic_list = []
        self.stat_list = []
        self.driver_name = ''
        self.driver_version = ''
        self.fcoe = {}
        self.fcoe['Interfaces'] = iface_section_list

        symb = False
        for line in content:
            line = line.strip()

            if 'Symbolic Name:' in line:
                symb = True

            if ':' in line:
                key, val = [s.strip() for s in line.split(': ', 1)]
                if not symb:
                    self.fcoe[key] = val
                else:
                    iface[key] = val

            if 'State:' in line:
                iface_section_list.append(iface)
                iface = {}

        self.driver_name = self.fcoe['Driver'].split(' ', 1)[0]
        self.driver_version = self.fcoe['Driver'].split(' ', 1)[-1]

        for iface in self.fcoe['Interfaces']:
            self.iface_list.append(iface['Symbolic Name'].split(' ')[-1])

        for nic in self.iface_list:
            self.nic_list.append(nic.split('.', 1)[0])

        for iface in self.fcoe['Interfaces']:
            self.stat_list.append(iface['State'])

    def get_stat_from_nic(self, nic):
        """
        Get 'State' of fcoe interface created on specified ethernet port.

        Parameter:
            nic (str): Ethernet port which provided by FCoE adapter.

        Return:
            str: Return fcoe status. When nic is not valid fcoe port,
                 raise ValueError.

        Raises:
            ValueError: When nic is not valid fcoe port
        """

        ret = ''
        self.__check_nic_valid(nic)

        for iface in self.fcoe['Interfaces']:
            if nic in iface['Symbolic Name']:
                ret = iface['State']
        return ret

    def get_host_from_nic(self, nic):
        """
        Get 'OS Device Name' for the specified ethernet port.

        Parameter:
            nic (str): Ethernet port which provided by FCoE adapter

        Return:
            str: Return fcoe host, which as 'OS Device Name' to display,
                 when nic is not valid fcoe port, raise ValueError

        Raises:
            ValueError: When nic is not valid fcoe port
        """

        ret = ''
        self.__check_nic_valid(nic)

        for iface in self.fcoe['Interfaces']:
            if nic in iface['Symbolic Name']:
                ret = iface['OS Device Name']
        return ret

    def __check_nic_valid(self, nic):
        BADNIC = "'%s' is NOT real FCoE port provided by HBA/CNA adapter" % nic

        if nic not in self.nic_list:
            raise ValueError(BADNIC)



"""
ModInfo
=======

The ModInfo combiner gathers all the ModInfoEach parsers into a dictionary
indexed by the module name.

"""

from insights.core.plugins import combiner
from insights.parsers.modinfo import ModInfoEach
from insights.parsers import SkipException


@combiner(ModInfoEach)
class ModInfo(dict):
    """
    Combiner for accessing all the modinfo outputs.

    Examples:
        >>> type(modinfo_obj)
        <class 'insights.combiners.modinfo.ModInfo'>
        >>> type(modinfo_obj.data['i40e'])
        <class 'insights.parsers.modinfo.ModInfoEach'>
        >>> modinfo_obj.data['i40e'].module_name
        'i40e'
        >>> modinfo_obj['i40e'].module_name
        'i40e'
        >>> modinfo_obj.data['i40e'].data['retpoline']
        'Y'
        >>> modinfo_obj.data['i40e'].module_version
        '2.3.2-k'
        >>> modinfo_obj.data['i40e'].module_path
        '/lib/modules/3.10.0-993.el7.x86_64/kernel/drivers/net/ethernet/intel/i40e/i40e.ko.xz'
        >>> "i40e" in modinfo_obj.retpoline_y
        True
        >>> "bnx2x" in modinfo_obj.retpoline_y
        False
        >>> "bnx2x" in modinfo_obj.retpoline_n
        True

    Raises:
        SkipException: When content is empty.

    Attributes:
        data (dict): A dictionary of parsed settings in format {name: ModInfoEach}.
        retpoline_y (set): A set of names of the modules with the attribute "retpoline: Y".
        retpoline_n (set): A set of names of the modules with the attribute "retpoline: N".
    """
    def __init__(self, modinfo):
        self.data = {}
        self.retpoline_y = set()
        self.retpoline_n = set()
        for m in modinfo:
            name = m.module_name
            self.data[name] = m
            if "retpoline" in m.data:
                r = m.data["retpoline"]
                if r == "Y":
                    self.retpoline_y.add(name)
                if r == "N":
                    self.retpoline_n.add(name)
        if not self.data:
            raise SkipException("No parsed contents")
        self.update(self.data)



from insights.tests import context_wrap
from insights.parsers.rsyslog_conf import RsyslogConf

RSYSLOG_CONF_0 = """
:fromhost-ip, regex, "10.0.0.[0-9]" /tmp/my_syslog.log
$ModLoad imtcp
$InputTCPServerRun 10514
$template SpiceTmpl,"%TIMESTAMP%.%TIMESTAMP:::date-subseconds% %syslogtag% %syslogseverity-text%:%msg:::sp-if-no-1st-sp%%msg:::drop-last-lf%\\n"
$WorkDirectory /var/opt/rsyslog # where to place spool files
""".strip()

RSYSLOG_CONF_1 = r"""
# Provides TCP syslog reception
#$ModLoad imtcp.so
#$InputTCPServerRun 514
:msg, regex, "\/vob\/.*\.[cpp|c|java]" /var/log/appMessages
""".strip()


def test_rsyslog_conf_0():
    ctx = context_wrap(RSYSLOG_CONF_0)
    m = RsyslogConf(ctx)
    d = list(m)
    assert len(m) == 5
    assert len(d) == 5
    # Test configuration item detection in dictionary
    assert hasattr(m, 'config_items')
    assert isinstance(m.config_items, dict)
    assert 'ModLoad' in m.config_items
    assert m.config_items['ModLoad'] == 'imtcp'
    assert m.config_items['InputTCPServerRun'] == '10514'
    assert m.config_items['template'] == 'SpiceTmpl,"%TIMESTAMP%.%TIMESTAMP:::date-subseconds% %syslogtag% %syslogseverity-text%:%msg:::sp-if-no-1st-sp%%msg:::drop-last-lf%\\n"'
    assert 'ForwardSyslogHost' not in m.config_items
    # configuration items should not include the comment.
    assert 'WorkDirectory' in m.config_items
    assert m.config_items['WorkDirectory'] == '/var/opt/rsyslog'
    # Test configuration item accessor
    assert hasattr(m, 'config_val')
    assert m.config_val('ModLoad') == 'imtcp'
    assert m.config_val('ForwardSyslogHost', 'syslog.example.com') == 'syslog.example.com'


def test_rsyslog_conf_1():
    ctx = context_wrap(RSYSLOG_CONF_1)
    m = RsyslogConf(ctx)
    d = list(m)
    assert len(m) == 1
    assert len(d) == 1
    # Test that commented-out config items are not detected
    assert 'ModLoad' not in m.config_items
    assert 'InputTCPServerRun' not in m.config_items



import doctest
import pytest

from insights.parsers import SkipException
from insights.parsers import partitions
from insights.parsers.partitions import Partitions
from insights.tests import context_wrap


PARTITIONS_CONTENT = """
major minor  #blocks  name

   8       16 1384120320 sdb
   8        0    3701760 sda
   8       32 2621440000 sdc

 253        0 1384120320 dm-0
   8       48 2726297600 sdd
 253        1 2621440000 dm-1
   8       64 1384120320 sde
 253        2 2726297600 dm-2
   8       96 2621440000 sdg
   8       80   52428800 sdf
   8      112 2726297600 sdh
   8      128  209715200 sdi
   8      144 1384120320 sdj
 253        3   52428800 dm-3
 253        4  209715200 dm-4
   8      160 1384120320 sdk
   8      176 2621440000 sdl
   8      192 2621440000 sdm
 253        5    1048576 dm-5
 253        6   51379200 dm-6
   8      224 2726297600 sdo
   8      208 2726297600 sdn
   8      240   52428800 sdp
  65        0   52428800 sdq
  65       16  209715200 sdr
  65       32  209715200 sds
  65       48   52428800 sdt
  65       64  209715200 sdu
 253        7   15728640 dm-7
 253        8   16777216 dm-8
 253        9   31457280 dm-9
 253       10   10485760 dm-10
 253       11    4194304 dm-11
 253       12  104857600 dm-12
 253       13    5242880 dm-13
 253       14    6287360 dm-14
 253       15    3145728 dm-15
 253       16 2726281216 dm-16
 253       17 1069531136 dm-17
""".strip()

INVALID_PARTITIONS_CONTENT = """
major minor  #blocks  name

   8       16 1384120320
   8        0    3701760 sda
   8       32 2621440000 sdc
""".strip()

EMPTY_CONTENT = """
""".strip()

SAMPLE_INPUT = """
major minor  #blocks  name

   3     0   19531250 hda
   3     1     104391 hda1
   3     2   19422585 hda2
 253     0   22708224 dm-0
 253     1     524288 dm-1
""".strip()


def test_partitions():
    info = Partitions(context_wrap(PARTITIONS_CONTENT))
    assert info.partitions
    assert 'dm-2' in info
    dm_2 = info['dm-2']
    assert dm_2.get('major') == '253'
    assert dm_2.get('minor') == '2'
    assert dm_2.get('blocks') == '2726297600'
    assert 'dm-18' not in info
    p_list = [i for i in info]
    assert p_list[2] in info
    assert len(p_list) == 39
    assert info['dm-17'] == {'major': '253', 'minor': '17', 'blocks': '1069531136', 'name': 'dm-17'}


def test_partitions_invalid_data():
    output = Partitions(context_wrap(INVALID_PARTITIONS_CONTENT))
    assert len(output) == 2
    assert 'sda' in output
    assert 'sdc' in output


def test_empty_content():
    with pytest.raises(SkipException) as exc:
        Partitions(context_wrap(EMPTY_CONTENT))
    assert 'Empty content' in str(exc)


def test_partitions_doc_examples():
    env = {'partitions_info': Partitions(context_wrap(SAMPLE_INPUT))}
    failed, total = doctest.testmod(partitions, globs=env)
    assert failed == 0



import doctest
from insights.parsers import scsi_fwver
from insights.parsers.scsi_fwver import SCSIFWver
from insights.tests import context_wrap

SCSI_HOST0_PATH = "/sys/class/scsi_host/host0/fwrev"
SCSI_HOSTS_0 = """
2.02X12 (U3H2.02X12), sli-3
""".strip()

SCSI_HOST1_PATH = "/sys/class/scsi_host/host1/fwrev"
SCSI_HOSTS_1 = """
2.02X13 (U3H3.02X13), sli-3
""".strip()

SCSI_HOST2_PATH = "/sys/class/scsi_host/host2/fwrev"
SCSI_HOSTS_2 = """
2.02X11 (U3H1.02X11)
""".strip()


def test_scsi_fwver_host0():
    context = context_wrap(SCSI_HOSTS_0, SCSI_HOST0_PATH)
    r = SCSIFWver(context)
    assert r.scsi_host == 'host0'
    assert r.host_mode == ['2.02X12 (U3H2.02X12)', 'sli-3']
    assert r.data == {'host0': ['2.02X12 (U3H2.02X12)', 'sli-3']}


def test_scsi_fwver_host1():
    context = context_wrap(SCSI_HOSTS_1, SCSI_HOST1_PATH)
    r = SCSIFWver(context)
    assert r.scsi_host == 'host1'
    assert r.host_mode == ['2.02X13 (U3H3.02X13)', 'sli-3']
    assert r.data == {'host1': ['2.02X13 (U3H3.02X13)', 'sli-3']}


def test_scsi_fwver_host2():
    context = context_wrap(SCSI_HOSTS_2, SCSI_HOST2_PATH)
    r = SCSIFWver(context)
    assert r.scsi_host == 'host2'
    assert r.host_mode == ['2.02X11 (U3H1.02X11)']
    assert r.data == {'host2': ['2.02X11 (U3H1.02X11)']}


def test_scsi_fwver_doc_examples():
    env = {'scsi_obj': SCSIFWver(context_wrap(SCSI_HOSTS_0, SCSI_HOST0_PATH))}
    failed, total = doctest.testmod(scsi_fwver, globs=env)
    assert failed == 0



from insights.parsers.ceph_version import CephVersion
from insights.parsers.ceph_version import CephVersionError
from insights.tests import context_wrap
import pytest

CV1 = "ceph version 0.94.9-9.el7cp (b83334e01379f267fb2f9ce729d74a0a8fa1e92c)"
CV2 = ""
CV3 = "error"
CV4 = "ceph version 10.2.2-38.el7cp (b83334e01379f267fb2f9ce729d74a0a8fa1e92c)"
CV5 = "ceph version 1"
CV6 = "ceph version 1.2.3-5"
CV_5 = "ceph version 10.2.5 (c461ee19ecbc0c5c330aca20f7392c9a00730367)"
CV7 = "ceph version 10.2.5-37.el7cp (033f137cde8573cfc5a4662b4ed6a63b8a8d1464)"
CV8 = "ceph version 10.2.7-27.el7cp (abcd137cde8573cfc5a4662b4ed6a63b8a8kadf1)"
CV9 = "ceph version 12.2.5-59.el7cp (d4b9f17b56b3348566926849313084dd6efc2ca2)"


def test_ceph_version():
    with pytest.raises(CephVersionError) as error_context2:
        CephVersion(context_wrap(CV2))
    assert 'Empty Ceph Version Line' in str(error_context2)
    with pytest.raises(CephVersionError) as error_context3:
        CephVersion(context_wrap(CV3))
    assert 'Wrong Format Ceph Version' in str(error_context3)
    with pytest.raises(CephVersionError) as error_context5:
        CephVersion(context_wrap(CV5))
    assert 'Wrong Format Ceph Version' in str(error_context5)
    with pytest.raises(CephVersionError) as error_context6:
        CephVersion(context_wrap(CV6))
    assert 'No Mapping Release Version' in str(error_context6)

    ceph_version1 = CephVersion(context_wrap(CV1))
    assert ceph_version1.version == "1.3.3"
    assert ceph_version1.major == '1.3'
    assert ceph_version1.minor == "3"
    assert ceph_version1.downstream_release == "async 2"

    ceph_version4 = CephVersion(context_wrap(CV4))
    assert ceph_version4.version == "2.0"
    assert ceph_version4.major == '2'
    assert ceph_version4.minor == "0"
    assert ceph_version4.downstream_release == "0"

    ceph = CephVersion(context_wrap(CV_5))
    assert ceph.version == "2.2"
    assert ceph.major == '2'
    assert ceph.minor == '2'
    assert ceph.downstream_release == "0"

    ceph_version7 = CephVersion(context_wrap(CV7))
    assert ceph_version7.version == "2.2"
    assert ceph_version7.major == '2'
    assert ceph_version7.minor == "2"
    assert ceph_version7.downstream_release == "0"

    ceph_version8 = CephVersion(context_wrap(CV8))
    assert ceph_version8.version == "2.3"
    assert ceph_version8.major == '2'
    assert ceph_version8.minor == "3"
    assert ceph_version8.downstream_release == "0"

    ceph_version9 = CephVersion(context_wrap(CV9))
    assert ceph_version9.version == "3.1.1"
    assert ceph_version9.major == '3'
    assert ceph_version9.minor == "1"
    assert ceph_version9.downstream_release == "1"



from __future__ import print_function
from __future__ import absolute_import
# !/usr/bin/python
# Copyright (C) 2016 Red Hat, All rights reserved.
# AUTHORS: Alex Collins <alcollin@redhat.com>

from . import util
import string

""" Module for extracting output of dmsetup. """


def getMajorMinor(deviceName, dmsetupLs):
    """
    Given output of dmsetup ls this will return
    themajor:minor (block name) of the device deviceName
    """

    startingIndex = string.rindex(dmsetupLs, deviceName) + len(deviceName)
    endingIndex = string.index(dmsetupLs[startingIndex:], "\n") + startingIndex
    # trim the preceding tab and ()'s
    newStr = dmsetupLs[startingIndex + 2: endingIndex - 1]
    return newStr


def getDmsetupLs():
    cmd = ['dmsetup', 'ls']
    r = util.subp(cmd)
    if r.return_code != 0:
        print(r.stderr)
        return -1
    return r.stdout



"""
grubby - command ``/usr/sbin/grubby``
=====================================

This is a collection of parsers that all deal with the command ``grubby``.
Parsers included in this module are:

GrubbyDefaultIndex - command ``grubby --default-index``
-------------------------------------------------------

GrubbyDefaultKernel - command ``grubby --default-kernel``
---------------------------------------------------------
"""

from insights.specs import Specs
from insights.parsers import SkipException, ParseException
from insights import CommandParser
from insights import parser


@parser(Specs.grubby_default_index)
class GrubbyDefaultIndex(CommandParser):
    """
    This parser parses the output of command ``grubby --default-index``.

    The typical output of this command is::

        0

    Examples:
        >>> grubby_default_index.default_index
        0

    Raises:
        SkipException: When output is empty
        ParseException: When output is invalid

    Attributes:
        default_index (int): the numeric index of the current default boot entry, count from 0
    """
    def parse_content(self, content):
        if not content:
            raise SkipException('Empty output')
        if len(content) != 1 or not content[0].isdigit():
            raise ParseException('Invalid output: {0}', content)
        self.default_index = int(content[0])


@parser(Specs.grubby_default_kernel)
class GrubbyDefaultKernel(CommandParser):
    """
    This parser parses the output of command ``grubby --default-kernel``.

    The typical output of this command is::

        /boot/vmlinuz-2.6.32-573.el6.x86_64

    Examples:

        >>> grubby_default_kernel.default_kernel
        '/boot/vmlinuz-2.6.32-573.el6.x86_64'

    Raises:
        SkipException: When output is empty
        ParseException: When output is invalid

    Attributes:
        default_kernel(str): The default kernel name for next boot
    """
    def parse_content(self, content):
        if not content:
            raise SkipException('Empty output')
        if len(content) != 1:
            raise ParseException('Invalid output: {0}', content)
        self.default_kernel = content[0].strip()



from insights.parsers import ksmstate
from insights.tests import context_wrap

KSMSTATE0 = "0"
KSMSTATE1 = "1"


def test_is_running_0():
    ksm_info = ksmstate.is_running(context_wrap(KSMSTATE0))
    assert ksm_info.get('running') is False


def test_is_running_1():
    ksm_info = ksmstate.is_running(context_wrap(KSMSTATE1))
    assert ksm_info.get('running') is True



#!/usr/bin/env python
"""
NginxConfTree - Combiner for nginx configuration
=======================================================
This module models nginx configuration as a tree. It correctly handles include
directives by splicing individual document trees into their parents until one
document tree is left.

A DSL is provided to query the tree through a select function or brackets [].
The brackets allow a more conventional lookup feel but aren't quite as powerful
as using select directly.
"""
import os
import string
from insights import combiner, parser, run
from insights.core import ConfigCombiner, ConfigParser
from insights.parsr.query import eq
from insights.parsr import (Char, EOF, Forward, LeftCurly, Lift, LineEnd,
        RightCurly, Many, Number, OneLineComment, Parser, PosMarker, SemiColon,
        QuotedString, skip_none, String, WS, WSChar)
from insights.parsr.query import Directive, Entry, Section
from insights.specs import Specs


class EmptyQuotedString(Parser):
    def __init__(self, chars):
        super(EmptyQuotedString, self).__init__()
        single = Char("'") >> String(set(chars) - set("'"), "'", 0) << Char("'")
        double = Char('"') >> String(set(chars) - set('"'), '"', 0) << Char('"')
        self.add_child(single | double)

    def process(self, pos, data, ctx):
        return self.children[0].process(pos, data, ctx)


@parser(Specs.nginx_conf)
class _NginxConf(ConfigParser):
    def __init__(self, *args, **kwargs):
        def to_entry(name, attrs, body):
            if body == ";":
                return Directive(name=name.value, attrs=attrs, lineno=name.lineno, src=self)
            return Section(name=name.value, attrs=attrs, children=body, lineno=name.lineno, src=self)

        name_chars = string.ascii_letters + "_/"
        Stmt = Forward()
        Num = Number & (WSChar | LineEnd | SemiColon)
        Comment = OneLineComment("#").map(lambda x: None)
        BeginBlock = WS >> LeftCurly << WS
        EndBlock = WS >> RightCurly << WS
        Bare = String(set(string.printable) - (set(string.whitespace) | set("#;{}'\"")))
        Name = WS >> PosMarker(String(name_chars) | EmptyQuotedString(name_chars)) << WS
        Attr = WS >> (Num | Bare | QuotedString) << WS
        Attrs = Many(Attr)
        Block = BeginBlock >> Many(Stmt).map(skip_none) << EndBlock
        Stanza = (Lift(to_entry) * Name * Attrs * (Block | SemiColon)) | Comment
        Stmt <= WS >> Stanza << WS
        Doc = Many(Stmt).map(skip_none)
        self.Top = Doc + EOF
        super(_NginxConf, self).__init__(*args, **kwargs)

    def parse_doc(self, content):
        return Entry(children=self.Top("\n".join(content))[0], src=self)


@combiner(_NginxConf)
class NginxConfTree(ConfigCombiner):
    """
    Exposes nginx configuration through the parsr query interface.

    See the :py:class:`insights.core.ConfigComponent` class for example usage.
    """
    def __init__(self, confs):
        super(NginxConfTree, self).__init__(confs, "nginx.conf", eq("include"))

    @property
    def conf_path(self):
        return os.path.dirname(self.main.file_path)


def get_tree(root=None):
    """
    This is a helper function to get an nginx configuration component for your
    local machine or an archive. It's for use in interactive sessions.
    """
    return run(NginxConfTree, root=root).get(NginxConfTree)


if __name__ == "__main__":
    run(NginxConfTree, print_summary=True)



"""
LsPci - Command ``lspci -k``
============================

To parse the PCI device information gathered from the ``/sbin/lspci -k`` command.

"""
import re

from .. import LogFileOutput, parser, CommandParser, get_active_lines
from insights.specs import Specs


@parser(Specs.lspci)
class LsPci(CommandParser, LogFileOutput):
    """
    Class to parse the PCI device information gathered from the
    ``/sbin/lspci -k`` command.

    Typical output of the ``lspci -k`` command is::

        00:00.0 Host bridge: Intel Corporation 5500 I/O Hub to ESI Port (rev 13)
                Subsystem: Cisco Systems Inc Device 0101
        00:01.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 1 (rev 13)
                Kernel driver in use: pcieport
                Kernel modules: shpchp
        00:02.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 2 (rev 13)
                Kernel driver in use: pcieport
                Kernel modules: shpchp
        03:00.0 Network controller: Intel Corporation Centrino Advanced-N 6205 [Taylor Peak] (rev 34)
                Subsystem: Cisco Systems Inc Device 004a
                Kernel driver in use: ixgbe
                Kernel modules: ixgbe
        06:00.0 Ethernet controller: Intel Corporation 82598EB 10-Gigabit AF Dual Port Network Connection (rev 01)
                Subsystem: Cisco Systems Inc Device 004a
                Kernel driver in use: ixgbe
                Kernel modules: ixgbe

    Examples:
        >>> type(lspci)
        <class 'insights.parsers.lspci.LsPci'>
        >>> lspci.get("Intel Corporation")[0]['raw_message']
        '00:00.0 Host bridge: Intel Corporation 5500 I/O Hub to ESI Port (rev 13)'
        >>> len(lspci.get("Network controller"))
        1
        >>> "Centrino Advanced-N 6205" in lspci
        True
        >>> "0d:00.0" in lspci
        False
        >>> sorted(lspci.pci_dev_list)
        ['00:00.0', '00:01.0', '00:02.0', '03:00.0', '06:00.0']
        >>> lspci.pci_dev_details('00:00.0')['Subsystem']
        'Cisco Systems Inc Device 0101'
        >>> lspci.pci_dev_details('00:00.0')['Dev_Details']
        'Host bridge: Intel Corporation 5500 I/O Hub to ESI Port (rev 13)'

    Attributes:
        data (dict): Dict where the keys are the device number and values are
            details of the device.
        lines (list): List of details of each listed device, the same to the
            values of `self.data`

    """
    def parse_content(self, content):
        # Use all the defined scanners to search the log file, setting the
        # properties defined in the scanner.
        self.lines = [l for l in content if len(l) > 0 and l[0].isdigit()]
        for scanner in self.scanners:
            scanner(self)
        # Parse kernel driver lines
        self.data = {}
        bus_device_function = None
        bus_device_function_re = re.compile(r'[0-9a-f]+:[0-9a-f]+.[0-9a-f]+')

        fields = ["Subsystem", "Kernel driver in use", "Kernel modules"]

        for line in get_active_lines(content):
            parts = line.split()

            if bus_device_function_re.match(parts[0]):
                bus_device_function = parts[0]
                device_details = ' '.join(map(str, parts[1:]))
                self.data[bus_device_function] = {'Dev_Details': device_details.lstrip()}
            elif bus_device_function and (line.split(":")[0].strip() in fields):
                parts = line.split(':')
                self.data[bus_device_function][parts[0]] = parts[1].lstrip()

    def pci_dev_details(self, dev_name):
        """
        It will return the PCI device and it's details.

        Args:
            PCI Bus Device function number eg: '00:01:0'

        Returns:
            (dict): Returns device details along with 'Subsystem', 'Kernel Driver in Use', 'Kernel Modules'.
            Returns `None` if device doesn't exists
        """
        return self.data.get(dev_name, None)

    @property
    def pci_dev_list(self):
        """
        The list of PCI devices.
        """
        return self.data.keys()



"""
Microsoft SQL Server Database Engine configuration - file ``/var/opt/mssql/mssql.conf``
=======================================================================================

The Microsoft SQL Server configuration file is a standard '.ini' file and uses
the ``IniConfigfile`` class to read it.

Sample configuration::

    [sqlagent]
    enabled = false

    [EULA]
    accepteula = Y

    [memory]
    memorylimitmb = 3328

Examples:

    >>> conf.has_option('memory', 'memorylimitmb')
    True
    >>> conf.get('memory', 'memorylimitmb') == '3328'
    True
"""
from insights.specs import Specs
from .. import parser, IniConfigFile


@parser(Specs.mssql_conf)
class MsSQLConf(IniConfigFile):
    """Microsoft SQL Server Database Engine configuration parser class, based on
    the ``IniConfigFile`` class.
    """
    pass



"""
Date parsers
============

This module provides processing for the output of the ``date`` command in
various formats.

Date - command ``date``
-----------------------

Class ``Date`` parses the output of the ``date`` command.  Sample output of
this command looks like::

    Fri Jun 24 09:13:34 CST 2016

DateUTC - command ``date --utc``
--------------------------------

Class ``DateUTC`` parses the output of the ``date --utc`` command.  Output is
similar to the ``date`` command except that the `Timezone` column uses UTC.

All classes utilize the same base class ``DateParser`` so the following
examples apply to all classes in this module.

Examples:
    >>> from insights.parsers.date import Date, DateUTC
    >>> from insights.tests import context_wrap
    >>> date_content = "Mon May 30 10:49:14 CST 2016"
    >>> shared = {Date: Date(context_wrap(date_content))}
    >>> date_info = shared[Date]
    >>> date_info.data
    'Mon May 30 10:49:14 CST 2016'
    >>> date_info.datetime is not None
    True
    >>> date_info.timezone
    'CST'

    >>> date_content = "Mon May 30 10:49:14 UTC 2016"
    >>> shared = {DateUTC: DateUTC(context_wrap(date_content))}
    >>> date_info = shared[DateUTC]
    >>> date_info.data
    'Mon May 30 10:49:14 UTC 2016'
    >>> date_info.datetime
    datetime.datetime(2016, 5, 30, 10, 49, 14)
    >>> date_info.timezone
    'UTC'
"""

import six
import sys
from datetime import datetime

from .. import parser, get_active_lines, CommandParser
from insights.specs import Specs


class DateParseException(Exception):
    pass


class DateParser(CommandParser):
    """Base class implementing shared code."""

    def parse_content(self, content):
        """
        Parses the output of the ``date`` and ``date --utc`` command.

        Sample: Fri Jun 24 09:13:34 CST 2016
        Sample: Fri Jun 24 09:13:34 UTC 2016

        Attributes
        ----------
        datetime: datetime.datetime
            A native datetime.datetime of the parsed date string
        timezone: str
            The string portion of the date string containing the timezone

        Raises:
            DateParseException: Raised if any exception occurs parsing the
            content.
        """
        self.data = get_active_lines(content, comment_char="COMMAND>")[0]
        parts = self.data.split()
        if not len(parts) == 6:
            msg = "Expected six date parts.  Got [%s]"
            raise DateParseException(msg % self.data)
        try:
            self.timezone = parts[4]
            no_tz = ' '.join(parts[:4]) + ' ' + parts[-1]
            self.datetime = datetime.strptime(no_tz, '%a %b %d %H:%M:%S %Y')
        except:
            six.reraise(DateParseException, DateParseException(self.data), sys.exc_info()[2])


@parser(Specs.date)
class Date(DateParser):
    """
    Class to parse ``date`` command output.

    Sample: Fri Jun 24 09:13:34 CST 2016
    """
    pass


@parser(Specs.date_utc)
class DateUTC(DateParser):
    """
    Class to parse ``date --utc`` command output.

    Sample: Fri Jun 24 09:13:34 UTC 2016
    """
    pass



import doctest
import pytest
from insights.parsers import ssh_client_config as scc, SkipException
from insights.tests import context_wrap

SSH_CONFIG_INPUT = """
#   ProxyCommand ssh -q -W %h:%p gateway.example.com
#   RekeyLimit 1G 1h
#
# Uncomment this if you want to use .local domain
# Host *.local
#   CheckHostIP no
ProxyCommand ssh -q -W %h:%p gateway.example.com

Host *
    GSSAPIAuthentication yes
# If this option is set to yes then remote X11 clients will have full access
# to the original X11 display. As virtually no X11 client supports the untrusted
# mode correctly we set this to yes.
    ForwardX11Trusted yes
# Send locale-related environment variables
    SendEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
    SendEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
    SendEnv LC_IDENTIFICATION LC_ALL LANGUAGE
    SendEnv XMODIFIERS

Host proxytest
    HostName 192.168.122.2
"""

SSH_CONFIG_INPUT_EMPTY = """
#   ProxyCommand ssh -q -W %h:%p gateway.example.com
#   RekeyLimit 1G 1h
#
# Uncomment this if you want to use .local domain
# Host *.local
#   CheckHostIP no
# If this option is set to yes then remote X11 clients will have full access
# to the original X11 display. As virtually no X11 client supports the untrusted
# mode correctly we set this to yes.
# Send locale-related environment variables
"""


def test_ssh_client_config():
    etcsshconfig = scc.EtcSshConfig(context_wrap(SSH_CONFIG_INPUT))
    assert len(etcsshconfig.global_lines) == 1
    assert ('Host_*' in etcsshconfig.host_lines) is True
    assert etcsshconfig.host_lines['Host_*'][0].keyword == 'GSSAPIAuthentication'
    assert etcsshconfig.host_lines['Host_proxytest'][0].value == '192.168.122.2'
    foremansshconfig = scc.ForemanSshConfig(context_wrap(SSH_CONFIG_INPUT))
    assert len(foremansshconfig.global_lines) == 1
    assert ('Host_*' in foremansshconfig.host_lines) is True
    assert foremansshconfig.host_lines['Host_*'][0].keyword == 'GSSAPIAuthentication'
    assert foremansshconfig.host_lines['Host_proxytest'][0].value == '192.168.122.2'


def test_ssh_config_AB():
    with pytest.raises(SkipException):
        scc.ForemanProxySshConfig(context_wrap(SSH_CONFIG_INPUT_EMPTY))


def test_ssh_client_config_docs():
    env = {
        'etcsshconfig': scc.EtcSshConfig(context_wrap(SSH_CONFIG_INPUT)),
        'foremansshconfig': scc.ForemanSshConfig(context_wrap(SSH_CONFIG_INPUT)),
        'foreman_proxy_ssh_config': scc.ForemanProxySshConfig(context_wrap(SSH_CONFIG_INPUT))
    }
    failed, total = doctest.testmod(scc, globs=env)
    assert failed == 0



from insights.tests import context_wrap
from insights.parsers.rhn_entitlement_cert_xml import RHNCertConf

xml_content = """
<?xml version="1.0" encoding="UTF-8"?>
<rhn-cert version="0.1">
  <rhn-cert-field name="product">RHN-SATELLITE-001</rhn-cert-field>
  <rhn-cert-field name="owner">Clay's Precious Satellite</rhn-cert-field>
  <rhn-cert-field name="issued">2005-01-11 00:00:00</rhn-cert-field>
  <rhn-cert-field name="expires">2005-03-11 00:00:00</rhn-cert-field>
  <rhn-cert-field name="slots">30</rhn-cert-field>
  <rhn-cert-field name="provisioning-slots">30</rhn-cert-field>
  <rhn-cert-field name="nonlinux-slots">30</rhn-cert-field>
  <rhn-cert-field name="channel-families" quantity="10" family="rhel-cluster"/>
  <rhn-cert-field name="channel-families" quantity="30" family="rhel-ws-extras"/>
  <rhn-cert-field name="channel-families" quantity="10" family="rhel-gfs"/>
  <rhn-cert-field name="channel-families" quantity="10" family="rhel-es-extras"/>
  <rhn-cert-field name="channel-families" quantity="40" family="rhel-as"/>
  <rhn-cert-field name="channel-families" quantity="30" family="rhn-tools"/>
  <rhn-cert-field name="channel-families" quantity="102" flex="0" family="sam-rhel-server-6"/>
  <rhn-cert-field name="channel-families" quantity="102" flex="51" family="cf-tools-5-beta"/>
  <rhn-cert-field name="satellite-version">5.2</rhn-cert-field>
  <rhn-cert-field name="generation">2</rhn-cert-field>
  <rhn-cert-signature>
-----BEGIN PGP SIGNATURE-----
Version: Crypt::OpenPGP 1.03

iQBGBAARAwAGBQJCAG7yAAoJEJ5yna8GlHkysOkAn07qmlUrkGKs7/5yb8H/nboG
mhHkAJ9wdmqOeKfcBa3IUDL53oNMEBP/dg==
=0Kv7
-----END PGP SIGNATURE-----
</rhn-cert-signature>
</rhn-cert>
"""


def test_match():
    result = RHNCertConf(context_wrap(xml_content, path='/etc/sysconfig/rhn/rhn_entitlement_cert_xml'))
    assert result.get("product") == "RHN-SATELLITE-001"
    assert result.get("channel-families").get('rhel-cluster') == {'quantity': '10'}
    assert result.file_name == 'rhn_entitlement_cert_xml'



from insights.parsers.ceph_version import CephVersion as CephV
from insights.parsers.ceph_insights import CephInsights
from insights.parsers.ceph_cmd_json_parsing import CephReport
from insights.combiners import ceph_version
from insights.combiners.ceph_version import CephVersion
from insights.tests import context_wrap
import doctest

CEPH_INSIGHTS = """
{
  "version": {
    "release": 12,
    "major": 2,
    "full": "ceph version 12.2.8-52.el7cp (3af3ca15b68572a357593c261f95038d02f46201) luminous (stable)",
    "minor": 8
  }
}
""".strip()

CEPH_VERSION = "ceph version 12.2.8-52.el7cp (3af3ca15b68572a357593c261f95038d02f46201) luminous (stable)".strip()

CEPH_REPORT = """
report 1188805303
{
    "cluster_fingerprint": "0d5f8f7a-0241-4a2e-8401-9dcf37a1039b",
    "version": "12.2.8-52.el7cp",
    "commit": "3af3ca15b68572a357593c261f95038d02f46201",
    "timestamp": "2019-06-05 23:33:08.514032",
    "tag": "",
    "health": {
        "checks": {
            "POOL_APP_NOT_ENABLED": {
                "severity": "HEALTH_WARN",
                "summary": {
                    "message": "application not enabled on 1 pool(s)"
                },
                "detail": [
                    {
                        "message": "application not enabled on pool 'pool-a'"
                    },
                    {
                        "message": "use 'ceph osd pool application enable <pool-name> <app-name>', where <app-name> is 'cephfs', 'rbd', 'rgw', or freeform for custom applications."
                    }
                ]
            },
            "MON_DOWN": {
                "severity": "HEALTH_WARN",
                "summary": {
                    "message": "1/3 mons down, quorum ceph2,ceph_1"
                },
                "detail": [
                    {
                        "message": "mon.ceph3 (rank 0) addr 10.72.37.76:6789/0 is down (out of quorum)"
                    }
                ]
            }
        },
        "status": "HEALTH_WARN",
        "summary": [
            {
                "severity": "HEALTH_WARN",
                "summary": "'ceph health' JSON format has changed in luminous. If you see this your monitoring system is scraping the wrong fields. Disable this with 'mon health preluminous compat warning = false'"
            }
        ],
        "overall_status": "HEALTH_WARN",
        "detail": [
            "'ceph health' JSON format has changed in luminous. If you see this your monitoring system is scraping the wrong fields. Disable this with 'mon health preluminous compat warning = false'"
        ]
    },
    "monmap_first_committed": 1,
    "monmap_last_committed": 1,
    "quorum": [
        1,
        2
    ],
    "osdmap_first_committed": 1,
    "osdmap_last_committed": 92
}
""".strip()


def test_ceph_version():
    cv = CephV(context_wrap(CEPH_VERSION))
    ret = CephVersion(cv, None, None)
    assert ret.version == "3.2"
    assert ret.major == "3"
    assert ret.minor == "2"
    assert ret.downstream_release == "0"
    assert ret.upstream_version["release"] == 12
    assert ret.upstream_version["major"] == 2
    assert ret.upstream_version["minor"] == 8


def test_ceph_version_2():
    cv = CephV(context_wrap(CEPH_VERSION))
    ci = CephInsights(context_wrap(CEPH_INSIGHTS))
    ret = CephVersion(cv, ci, None)
    assert ret.version == "3.2"
    assert ret.major == "3"
    assert ret.minor == "2"
    assert ret.downstream_release == "0"
    assert ret.upstream_version["release"] == 12
    assert ret.upstream_version["major"] == 2
    assert ret.upstream_version["minor"] == 8


def test_ceph_insights():
    ci = CephInsights(context_wrap(CEPH_INSIGHTS))
    ret = CephVersion(None, ci, None)
    assert ret.version == "3.2"
    assert ret.major == "3"
    assert ret.minor == "2"
    assert ret.downstream_release == "0"
    assert ret.upstream_version["release"] == 12
    assert ret.upstream_version["major"] == 2
    assert ret.upstream_version["minor"] == 8


def test_ceph_report():
    cr = CephReport(context_wrap(CEPH_REPORT))
    ret = CephVersion(None, None, cr)
    assert ret.version == "3.2"
    assert ret.major == "3"
    assert ret.minor == "2"
    assert ret.downstream_release == "0"
    assert ret.upstream_version["release"] == 12
    assert ret.upstream_version["major"] == 2
    assert ret.upstream_version["minor"] == 8


def test_ceph_version_doc_examples():
    env = {
        'cv': CephVersion(CephV(context_wrap(CEPH_VERSION)), None, None)
    }
    failed, total = doctest.testmod(ceph_version, globs=env)
    assert failed == 0



from insights.parsers.cib import CIB
from insights.tests import context_wrap

CIB_CONFIG = """
    <cib crm_feature_set="3.0.9" validate-with="pacemaker-2.3" have-quorum="1" dc-uuid="4">
      <configuration>
        <crm_config>
          <cluster_property_set id="cib-bootstrap-options">
            <nvpair id="cib-bootstrap-options-have-watchdog" name="have-watchdog" value="false"/>
            <nvpair id="cib-bootstrap-options-no-quorum-policy" name="no-quorum-policy" value="freeze"/>
          </cluster_property_set>
        </crm_config>
        <nodes>
          <node id="1" uname="foo"/>
          <node id="2" uname="bar"/>
          <node id="3" uname="baz"/>
        </nodes>
        <resources>
          <clone id="dlm-clone">
          </clone>
        </resources>
        <constraints>
          <rsc_order first="dlm-clone" first-action="start" id="order-dlm-clone-clvmd-clone-mandatory" then="clvmd-clone" then-action="start"/>
          <rsc_colocation id="colocation-clvmd-clone-dlm-clone-INFINITY" rsc="clvmd-clone" score="INFINITY" with-rsc="dlm-clone"/>
        </constraints>
      </configuration>
    </cib>
"""


def test_cib():
    cib = CIB(context_wrap(CIB_CONFIG))
    assert cib is not None
    assert cib.nodes == ['foo', 'bar', 'baz']



"""
Handle compressed filesystems
This replicates functionality that 'archivemount' would provide
"""
import logging
import tempfile
import tarfile
import os
import shutil

logger = logging.getLogger(__name__)


class InsightsCompressedFile(object):

    """
    This class handles uncompressing and mounting compressed filesystems
    """

    def __init__(self, compressed_file_location=None):
        self.compressed_file_location = compressed_file_location
        self.tmp_dir = tempfile.mkdtemp(prefix='/var/tmp/')
        self.is_file = os.path.isfile(self.compressed_file_location)
        self.is_tarfile = (tarfile.is_tarfile(self.compressed_file_location)
                            if self.is_file else False)
        if self.is_file and self.is_tarfile:
            try:
                tar = tarfile.open(self.compressed_file_location)
                tar.extractall(path=self.tmp_dir)
                tar.close()
                logger.debug("Compressed filesystem %s extracted to %s",
                    self.compressed_file_location, self.tmp_dir)
            except:
                logger.debug("Invalid compressed tar filesystem provided. "
                    "Could not extract contents.")
        else:
            logger.debug("Invalid compressed tar filesystem provided.")

    def cleanup_temp_filesystem(self):
        """
        Cleanup the temporary directory
        """
        logger.debug("Deleting compressed file extraction directory: " + self.tmp_dir)
        shutil.rmtree(self.tmp_dir, True)

    def get_filesystem_path(self):
        """
        Get the filesystem path, where it was extracted to
        """
        return self.tmp_dir



from glob import glob
from insights.client.connection import InsightsConnection
from insights.client.constants import InsightsConstants as constants
from insights.util.canonical_facts import get_canonical_facts
from logging import getLogger
from platform import linux_distribution
from re import findall
from subprocess import Popen, PIPE, STDOUT
from sys import exit

OSCAP_RESULTS_OUTPUT = '/tmp/oscap_results.xml'
NONCOMPLIANT_STATUS = 2
COMPLIANCE_CONTENT_TYPE = 'application/vnd.redhat.compliance.something+tgz'
POLICY_FILE_LOCATION = '/usr/share/xml/scap/ssg/content/'
logger = getLogger(__name__)


class ComplianceClient:
    def __init__(self, config):
        self.config = config
        self.conn = InsightsConnection(config)
        self.hostname = get_canonical_facts().get('fqdn', '')

    def oscap_scan(self):
        policies = self.get_policies()
        if not policies:
            logger.error("ERROR: System does not exist!\n")
            exit(constants.sig_kill_bad)
        profile_ref_id = [policy['attributes']['ref_id'] for policy in policies][0]
        scap_policy_xml = self.find_scap_policy(profile_ref_id)
        self.run_scan(profile_ref_id, scap_policy_xml)
        return OSCAP_RESULTS_OUTPUT, COMPLIANCE_CONTENT_TYPE

    # TODO: Not a typo! This endpoint gives OSCAP policies, not profiles
    # We need to update compliance-backend to fix this
    def get_policies(self):
        response = self.conn.session.get("https://{0}/compliance/profiles".format(self.config.base_url), params={'hostname': self.hostname})
        if response.status_code == 200:
            return response.json()['data']
        else:
            return []

    def os_release(self):
        _, version, _ = linux_distribution()
        return findall("^[6-8]", version)[0]

    def profile_files(self):
        return glob("{0}*rhel{1}*.xml".format(POLICY_FILE_LOCATION, self.os_release()))

    def find_scap_policy(self, profile_ref_id):
        grep = Popen(["grep", profile_ref_id] + self.profile_files(), stdout=PIPE, stderr=STDOUT)
        if grep.wait():
            logger.error('ERROR: XML profile file not found matching ref_id {0}\n{1}\n'.format(profile_ref_id, grep.stderr.read()))
            exit(constants.sig_kill_bad)
        filenames = findall('/usr/share/xml/scap/.+xml', grep.stdout.read().decode('utf-8'))
        if not filenames:
            logger.error('ERROR: No XML profile files found matching ref_id {0}\n{1}\n'.format(profile_ref_id, ' '.join(filenames)))
            exit(constants.sig_kill_bad)
        return filenames[0]

    def run_scan(self, profile_ref_id, policy_xml):
        logger.info("Running scan... this may take a while")
        oscap = Popen(["oscap", "xccdf", "eval", "--profile", profile_ref_id, "--results", OSCAP_RESULTS_OUTPUT, policy_xml], stdout=PIPE, stderr=STDOUT)
        if oscap.wait() and oscap.wait() != NONCOMPLIANT_STATUS:
            logger.error("ERROR: Scan failed")
            logger.error(oscap.stderr.read())
            exit(constants.sig_kill_bad)



"""
freeipa_healthcheck_log - File ``/var/log/ipa/healthcheck/healthcheck.log``
===========================================================================

This module provides plugins access to file ``/var/log/ipa/healthcheck/healthcheck.log``

This file contains a list of items, each item representing a check made on the local IPA server.
The file can be either single-line or multi-line indented.

Typical content of file ``/var/log/ipa/healthcheck/healthcheck.log`` is::

    [
      {
        "source": "ipahealthcheck.dogtag.ca",
        "check": "DogtagCertsConfigCheck",
        "severity": 0,
        "uuid": "57f7d2c9-f71f-4c2f-b831-9e30250b18b2",
        "when": "20190726213428Z",
        "duration": "0.150333",
        "kw": {
          "key": "subsystemCert cert-pki-ca",
          "configfile": "/var/lib/pki/pki-tomcat/conf/ca/CS.cfg"
        }
      },
      {
        "source": "ipahealthcheck.ipa.certs",
        "check": "IPACertmongerExpirationCheck",
        "severity": 0,
        "uuid": "afd047c2-d78b-4c5c-b3a4-f68578ee1595",
        "when": "20190726213429Z",
        "duration": "0.007105",
        "kw": {
          "key": "20190620165230"
        }
      },
      {
        "source": "ipahealthcheck.ds.replication",
        "check": "ReplicationConflictCheck",
        "severity": 0,
        "uuid": "99c0a513-447c-46f1-95ea-058fc0db6075",
        "when": "20190726213429Z",
        "duration": "0.001409",
        "kw": {}
      },
      {
        "source": "ipahealthcheck.ipa.files",
        "check": "IPAFileNSSDBCheck",
        "severity": 0,
        "uuid": "7e8a7739-c4d6-4b97-b602-9f9e981f793c",
        "when": "20190726213432Z",
        "duration": "0.000765",
        "kw": {
          "key": "_etc_pki_pki-tomcat_alias_cert9.db_owner",
          "type": "owner",
          "path": "/etc/pki/pki-tomcat/alias/cert9.db"
        }
      },
      {
        "source": "ipahealthcheck.ipa.topology",
        "check": "IPATopologyDomainCheck",
        "severity": 0,
        "uuid": "c5ee8ee1-98d1-4696-bbf1-8243677b8918",
        "when": "20190726213432Z",
        "duration": "0.019070",
        "kw": {
          "suffix": "ca"
        }
      },
      {
        "source": "ipahealthcheck.system.filesystemspace",
        "check": "FileSystemSpaceCheck",
        "severity": 2,
        "uuid": "4d1c71c5-cc37-4ebf-b53c-34f4e4534437",
        "when": "20190726213432Z",
        "duration": null,
        "kw": {
          "msg": "/var/lib/dirsrv/: free space percentage under threshold: 1% < 20%",
          "store": "/var/lib/dirsrv/",
          "percent_free": 1,
          "threshold": 20
        }
      }
    ]

The list of errors can be accessed via the errors property.

Examples:

    >>> len(healthcheck.errors)
    1
    >>> healthcheck.errors[0]['check'] == 'FileSystemSpaceCheck'
    True

"""

from insights import parser
from insights import JSONParser
from insights.specs import Specs


@parser(Specs.freeipa_healthcheck_log)
class FreeIPAHealthCheckLog(JSONParser):
    """Parses the content of file ``/var/log/ipa/healthcheck/healthcheck.log``."""

    @property
    def errors(self):
        """ list: errors in healthcheck log."""
        return [entry for entry in self.data if entry['severity'] > 0]



from insights.parsr.examples.multipath_conf import loads


EXAMPLE = """
# This is a basic configuration file with some examples, for device mapper
# multipath.
#
# For a complete list of the default configuration values, run either
# multipath -t
# or
# multipathd show config
#
# For a list of configuration options with descriptions, see the multipath.conf
# man page

## By default, devices with vendor = "IBM" and product = "S/390.*" are
## blacklisted. To enable mulitpathing on these devies, uncomment the
## following lines.
#blacklist_exceptions {
#	device {
#		vendor	"IBM"
#		product	"S/390.*"
#	}
#}

## Use user friendly names, instead of using WWIDs as names.
defaults {
    user_friendly_names yes
    find_multipaths yes
}
##
## Here is an example of how to configure some standard options.
##
#
#defaults {
#	udev_dir		/dev
#	polling_interval 	10
#	selector		"round-robin 0"
#	path_grouping_policy	multibus
#	prio			alua
#	path_checker		readsector0
#	rr_min_io		100
#	max_fds			8192
#	rr_weight		priorities
#	failback		immediate
#	no_path_retry		fail
#	user_friendly_names	yes
#}
##
## The wwid line in the following blacklist section is shown as an example
## of how to blacklist devices by wwid.  The 2 devnode lines are the
## compiled in default blacklist. If you want to blacklist entire types
## of devices, such as all scsi devices, you should use a devnode line.
## However, if you want to blacklist specific devices, you should use
## a wwid line.  Since there is no guarantee that a specific device will
## not change names on reboot (from /dev/sda to /dev/sdb for example)
## devnode lines are not recommended for blacklisting specific devices.
##
#blacklist {
#       wwid 26353900f02796769
#	devnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"
#	devnode "^hd[a-z]"
#}
#multipaths {
#	multipath {
#		wwid			3600508b4000156d700012000000b0000
#		alias			yellow
#		path_grouping_policy	multibus
#		path_checker		readsector0
#		path_selector		"round-robin 0"
#		failback		manual
#		rr_weight		priorities
#		no_path_retry		5
#	}
#	multipath {
#		wwid			1DEC_____321816758474
#		alias			red
#	}
#}
#devices {
#	device {
#		vendor			"COMPAQ  "
#		product			"HSV110 (C)COMPAQ"
#		path_grouping_policy	multibus
#		path_checker		readsector0
#		path_selector		"round-robin 0"
#		hardware_handler	"0"
#		failback		15
#		rr_weight		priorities
#		no_path_retry		queue
#	}
#	device {
#		vendor			"COMPAQ  "
#		product			"MSA1000         "
#		path_grouping_policy	multibus
#	}
#}
#"""

CONF = """
blacklist {
       device {
               vendor  "IBM"
               product "3S42"       #DS4200 Product 10
       }
       device {
               vendor  "HP"
               product "*"
       }
}""".strip()


MULTIPATH_CONF_INFO = """
defaults {
       udev_dir                /dev
       path_selector           "round-robin 0"
       user_friendly_names     yes
}

multipaths {
       multipath {
               alias                   yellow
               path_grouping_policy    multibus
       }
       multipath {
               wwid                    1DEC_____321816758474
               alias                   red
       }
}

devices {
       device {
               path_selector           "round-robin 0"
               no_path_retry           queue
       }
       device {
               vendor                  "COMPAQ  "
               path_grouping_policy    multibus
       }
}

blacklist {
       wwid 26353900f02796769
       devnode "^hd[a-z]"
}

""".strip()


def test_multipath_example():
    res = loads(EXAMPLE)
    assert res["defaults"]["user_friendly_names"].value == "yes"


def test_multipath_conf():
    res = loads(CONF)
    assert res["blacklist"]["device"][0]["product"].value == "3S42"


def xtest_multipath_conf_info():
    res = loads(MULTIPATH_CONF_INFO)
    assert res["defaults"]["path_selector"].value == "round-robin 0"
    assert res["multipaths"]["multipath"][1]["wwid"].value == "1DEC_____321816758474"



from insights.parsers import ParseException
from insights.parsers import vdo_status
from insights.parsers.vdo_status import VDOStatus
from insights.tests import context_wrap
import pytest
import doctest


INPUT_EXP = """
VDO status 2019-07-27 04:40:40-04:00
rdma-qe-04.lab.bos.redhat.com
""".strip()


INPUT_SIMPLE_EXP = """
VDO status:
  Date: '2019-07-27 04:40:40-04:00'
  Node: rdma-qe-04.lab.bos.redhat.com
Kernel module:
  Loaded: true
  Name: kvdo
  Version information:
    kvdo version: 6.1.0.153
Configuration:
  File: /etc/vdoconf.yml
  Last modified: '2019-07-26 05:07:48'
V??DOs:
  vdo1:
     XX: XX
  vdo2:
     XX: XX

"""


INPUT_STATUS_SIMPLE = """
VDO status:
  Date: '2019-07-27 04:40:40-04:00'
  Node: rdma-qe-04.lab.bos.redhat.com
Kernel module:
  Loaded: true
  Name: kvdo
  Version information:
    kvdo version: 6.1.0.153
Configuration:
  File: /etc/vdoconf.yml
  Last modified: '2019-07-26 05:07:48'
VDOs:
  vdo1:
    Acknowledgement threads: 1
    Activate: enabled
    Slab size: 2G
    VDO statistics:
      /dev/mapper/vdo1:
        block size: 4096
        data blocks used: 161863
        logical blocks: 844568
        logical blocks used: 844502
        overhead blocks used: 728175
        physical blocks: 1572864
        slab count: 26
  vdo2:
    Acknowledgement threads: 1
    Activate: enabled
    Slab size: 128M
    VDO statistics:
      /dev/mapper/vdo2:
        block size: 4096
        data blocks used: 161861
        logical blocks: 844562
        logical blocks used: 0
        overhead blocks used: 728172
        physical blocks: 1572861
        slab count: 26
"""

INPUT_EMPTY = """
VDO status:
  Date: '2019-07-25 01:38:47-04:00'
  Node: rdma-dev-09.lab.bos.redhat.com
Kernel module:
  Loaded: false
  Name: kvdo
  Version information:
    kvdo version: 6.1.0.153
Configuration:
  File: does not exist
  Last modified: not available
VDOs: {}
""".strip()


INPUT_STATUS_FULL = """
VDO status:
  Date: '2019-07-24 20:48:16-04:00'
  Node: dell-m620-10.rhts.gsslab.pek2.redhat.com
Kernel module:
  Loaded: true
  Name: kvdo
  Version information:
    kvdo version: 6.1.0.153
Configuration:
  File: /etc/vdoconf.yml
  Last modified: '2019-07-24 20:47:59'
VDOs:
  vdo1:
    Acknowledgement threads: 1
    Activate: enabled
    Bio rotation interval: 64
    Bio submission threads: 4
    Block map cache size: 128M
    Block map period: 16380
    Block size: 4096
    CPU-work threads: 2
    Compression: enabled
    Configured write policy: auto
    Deduplication: enabled
    Device mapper status: 0 8370216 vdo /dev/sda3 albserver online cpu=2,bio=4,ack=1,bioRotationInterval=64
    Emulate 512 byte: disabled
    Hash zone threads: 1
    Index checkpoint frequency: 0
    Index memory setting: 0.25
    Index parallel factor: 0
    Index sparse: disabled
    Index status: online
    Logical size: 4185108K
    Logical threads: 1
    Physical size: 7G
    Physical threads: 1
    Read cache: disabled
    Read cache size: 0M
    Slab size: 2G
    Storage device: /dev/sda3
    VDO statistics:
      /dev/mapper/vdo1:
        1K-blocks: 7340032
        1K-blocks available: 4191472
        1K-blocks used: 3148560
        512 byte emulation: false
        KVDO module bios used: 74572
        KVDO module bytes used: 851421880
        KVDO module peak bio count: 74860
        KVDO module peak bytes used: 851423752
        bios acknowledged discard: 0
        bios acknowledged flush: 0
        bios acknowledged fua: 0
        bios acknowledged partial discard: 0
        bios acknowledged partial flush: 0
        bios acknowledged partial fua: 0
        bios acknowledged partial read: 0
        bios acknowledged partial write: 0
        bios acknowledged read: 261
        bios acknowledged write: 0
        bios in discard: 0
        bios in flush: 0
        bios in fua: 0
        bios in partial discard: 0
        bios in partial flush: 0
        bios in partial fua: 0
        bios in partial read: 0
        bios in partial write: 0
        bios in progress discard: 0
        bios in progress flush: 0
        bios in progress fua: 0
        bios in progress read: 0
        bios in progress write: 0
        bios in read: 261
        bios in write: 0
        bios journal completed discard: 0
        bios journal completed flush: 0
        bios journal completed fua: 0
        bios journal completed read: 0
        bios journal completed write: 0
        bios journal discard: 0
        bios journal flush: 0
        bios journal fua: 0
        bios journal read: 0
        bios journal write: 0
        bios meta completed discard: 0
        bios meta completed flush: 0
        bios meta completed fua: 0
        bios meta completed read: 3
        bios meta completed write: 65
        bios meta discard: 0
        bios meta flush: 1
        bios meta fua: 1
        bios meta read: 3
        bios meta write: 65
        bios out completed discard: 0
        bios out completed flush: 0
        bios out completed fua: 0
        bios out completed read: 0
        bios out completed write: 0
        bios out discard: 0
        bios out flush: 0
        bios out fua: 0
        bios out read: 0
        bios out write: 0
        bios page cache completed discard: 0
        bios page cache completed flush: 0
        bios page cache completed fua: 0
        bios page cache completed read: 0
        bios page cache completed write: 0
        bios page cache discard: 0
        bios page cache flush: 0
        bios page cache fua: 0
        bios page cache read: 0
        bios page cache write: 0
        block map cache pressure: 0
        block map cache size: 134217728
        block map clean pages: 0
        block map dirty pages: 0
        block map discard required: 0
        block map failed pages: 0
        block map failed reads: 0
        block map failed writes: 0
        block map fetch required: 0
        block map flush count: 0
        block map found in cache: 0
        block map free pages: 32768
        block map incoming pages: 0
        block map outgoing pages: 0
        block map pages loaded: 0
        block map pages saved: 0
        block map read count: 0
        block map read outgoing: 0
        block map reclaimed: 0
        block map wait for page: 0
        block map write count: 0
        block size: 4096
        completed recovery count: 0
        compressed blocks written: 0
        compressed fragments in packer: 0
        compressed fragments written: 0
        current VDO IO requests in progress: 0
        current dedupe queries: 0
        data blocks used: 0
        dedupe advice stale: 0
        dedupe advice timeouts: 0
        dedupe advice valid: 0
        entries indexed: 0
        flush out: 0
        instance: 0
        invalid advice PBN count: 0
        journal blocks batching: 0
        journal blocks committed: 0
        journal blocks started: 0
        journal blocks writing: 0
        journal blocks written: 0
        journal commits requested count: 0
        journal disk full count: 0
        journal entries batching: 0
        journal entries committed: 0
        journal entries started: 0
        journal entries writing: 0
        journal entries written: 0
        logical blocks: 1046277
        logical blocks used: 0
        maximum VDO IO requests in progress: 9
        maximum dedupe queries: 0
        no space error count: 0
        operating mode: normal
        overhead blocks used: 787140
        physical blocks: 1835008
        posts found: 0
        posts not found: 0
        queries found: 0
        queries not found: 0
        read cache accesses: 0
        read cache data hits: 0
        read cache hits: 0
        read only error count: 0
        read-only recovery count: 0
        recovery progress (%): N/A
        reference blocks written: 0
        release version: 131337
        saving percent: N/A
        slab count: 2
        slab journal blocked count: 0
        slab journal blocks written: 0
        slab journal disk full count: 0
        slab journal flush count: 0
        slab journal tail busy count: 0
        slab summary blocks written: 0
        slabs opened: 0
        slabs reopened: 0
        updates found: 0
        updates not found: 0
        used percent: 42
        version: 26
        write amplification ratio: 0.0
        write policy: sync
  vdo2:
    Acknowledgement threads: 1
    Activate: enabled
    Bio rotation interval: 64
    Bio submission threads: 4
    Block map cache size: 128M
    Block map period: 16380
    Block size: 4096
    CPU-work threads: 2
    Compression: enabled
    Configured write policy: auto
    Deduplication: enabled
    Device mapper status: 0 4183912 vdo /dev/sda5 albserver online cpu=2,bio=4,ack=1,bioRotationInterval=64
    Emulate 512 byte: disabled
    Hash zone threads: 1
    Index checkpoint frequency: 0
    Index memory setting: 0.25
    Index parallel factor: 0
    Index sparse: disabled
    Index status: online
    Logical size: 2091956K
    Logical threads: 1
    Physical size: 5G
    Physical threads: 1
    Read cache: disabled
    Read cache size: 0M
    Slab size: 2G
    Storage device: /dev/sda5
    VDO statistics:
      /dev/mapper/vdo2:
        1K-blocks: 5242880
        1K-blocks available: 2095736
        1K-blocks used: 3147144
        512 byte emulation: false
        KVDO module bios used: 74572
        KVDO module bytes used: 851421880
        KVDO module peak bio count: 74860
        KVDO module peak bytes used: 851423752
        bios acknowledged discard: 0
        bios acknowledged flush: 0
        bios acknowledged fua: 0
        bios acknowledged partial discard: 0
        bios acknowledged partial flush: 0
        bios acknowledged partial fua: 0
        bios acknowledged partial read: 0
        bios acknowledged partial write: 0
        bios acknowledged read: 265
        bios acknowledged write: 0
        bios in discard: 0
        bios in flush: 0
        bios in fua: 0
        bios in partial discard: 0
        bios in partial flush: 0
        bios in partial fua: 0
        bios in partial read: 0
        bios in partial write: 0
        bios in progress discard: 0
        bios in progress flush: 0
        bios in progress fua: 0
        bios in progress read: 0
        bios in progress write: 0
        bios in read: 265
        bios in write: 0
        bios journal completed discard: 0
        bios journal completed flush: 0
        bios journal completed fua: 0
        bios journal completed read: 0
        bios journal completed write: 0
        bios journal discard: 0
        bios journal flush: 0
        bios journal fua: 0
        bios journal read: 0
        bios journal write: 0
        bios meta completed discard: 0
        bios meta completed flush: 0
        bios meta completed fua: 0
        bios meta completed read: 4
        bios meta completed write: 65
        bios meta discard: 0
        bios meta flush: 1
        bios meta fua: 1
        bios meta read: 4
        bios meta write: 65
        bios out completed discard: 0
        bios out completed flush: 0
        bios out completed fua: 0
        bios out completed read: 0
        bios out completed write: 0
        bios out discard: 0
        bios out flush: 0
        bios out fua: 0
        bios out read: 0
        bios out write: 0
        bios page cache completed discard: 0
        bios page cache completed flush: 0
        bios page cache completed fua: 0
        bios page cache completed read: 0
        bios page cache completed write: 0
        bios page cache discard: 0
        bios page cache flush: 0
        bios page cache fua: 0
        bios page cache read: 0
        bios page cache write: 0
        block map cache pressure: 0
        block map cache size: 134217728
        block map clean pages: 0
        block map dirty pages: 0
        block map discard required: 0
        block map failed pages: 0
        block map failed reads: 0
        block map failed writes: 0
        block map fetch required: 0
        block map flush count: 0
        block map found in cache: 0
        block map free pages: 32768
        block map incoming pages: 0
        block map outgoing pages: 0
        block map pages loaded: 0
        block map pages saved: 0
        block map read count: 0
        block map read outgoing: 0
        block map reclaimed: 0
        block map wait for page: 0
        block map write count: 0
        block size: 4096
        completed recovery count: 0
        compressed blocks written: 0
        compressed fragments in packer: 0
        compressed fragments written: 0
        current VDO IO requests in progress: 0
        current dedupe queries: 0
        data blocks used: 1
        dedupe advice stale: 0
        dedupe advice timeouts: 0
        dedupe advice valid: 0
        entries indexed: 0
        flush out: 0
        instance: 1
        invalid advice PBN count: 0
        journal blocks batching: 0
        journal blocks committed: 0
        journal blocks started: 0
        journal blocks writing: 0
        journal blocks written: 0
        journal commits requested count: 0
        journal disk full count: 0
        journal entries batching: 0
        journal entries committed: 0
        journal entries started: 0
        journal entries writing: 0
        journal entries written: 0
        logical blocks: 522989
        logical blocks used: 3
        maximum VDO IO requests in progress: 7
        maximum dedupe queries: 0
        no space error count: 0
        operating mode: normal
        overhead blocks used: 786786
        physical blocks: 1310720
        posts found: 0
        posts not found: 0
        queries found: 0
        queries not found: 0
        read cache accesses: 0
        read cache data hits: 0
        read cache hits: 0
        read only error count: 0
        read-only recovery count: 0
        recovery progress (%): N/A
        reference blocks written: 0
        release version: 131337
        saving percent: N/A
        slab count: 1
        slab journal blocked count: 0
        slab journal blocks written: 0
        slab journal disk full count: 0
        slab journal flush count: 0
        slab journal tail busy count: 0
        slab summary blocks written: 0
        slabs opened: 0
        slabs reopened: 0
        updates found: 0
        updates not found: 0
        used percent: 60
        version: 26
        write amplification ratio: 0.0
        write policy: sync
""".strip()


def test_vdo_status_simple():
    vdo = VDOStatus(context_wrap(INPUT_STATUS_SIMPLE))
    assert vdo.data['VDO status'] == {'Date': '2019-07-27 04:40:40-04:00', 'Node': 'rdma-qe-04.lab.bos.redhat.com'}
    assert vdo.data['VDOs']['vdo1']['Activate'] == 'enabled'
    assert vdo.data['VDO status']['Date'] == '2019-07-27 04:40:40-04:00'
    assert vdo.data['VDO status']['Node'] == 'rdma-qe-04.lab.bos.redhat.com'

    assert vdo.get_slab_size_of_vol('vdo1') == '2G'
    assert vdo.volumns == ['vdo1', 'vdo2']

    assert vdo.get_physical_blocks_of_vol('vdo1') == 1572864
    assert vdo.get_overhead_used_of_vol('vdo1') == 728175
    assert vdo.get_physical_used_of_vol('vdo1') == 161863
    assert vdo.get_physical_free_of_vol('vdo1') == 682826

    assert vdo.get_logical_blocks_of_vol('vdo1') == 844568
    assert vdo.get_logical_used_of_vol('vdo1') == 844502
    assert vdo.get_logical_free_of_vol('vdo1') == 66


def test_vdo_status_empty():
    vdo = VDOStatus(context_wrap(INPUT_EMPTY))
    assert vdo.data["VDOs"] == {}

    assert vdo.data["Configuration"]["File"] == "does not exist"
    assert vdo.data["Configuration"]["Last modified"] == "not available"


def test_vdo_status_full():
    vdo = VDOStatus(context_wrap(INPUT_STATUS_FULL))
    assert vdo.data["VDO status"]["Date"] == "2019-07-24 20:48:16-04:00"
    assert vdo.data["VDO status"]["Node"] == "dell-m620-10.rhts.gsslab.pek2.redhat.com"

    assert vdo.data["Kernel module"]["Name"] == "kvdo"
    assert vdo.data["Kernel module"]["Version information"]["kvdo version"] == "6.1.0.153"

    assert vdo.data["Configuration"]["File"] == "/etc/vdoconf.yml"
    assert vdo.data["Configuration"]["Last modified"] == "2019-07-24 20:47:59"

    assert vdo.data["VDOs"]["vdo1"]["Acknowledgement threads"] == 1
    assert vdo.data["VDOs"]["vdo1"]["Device mapper status"] == "0 8370216 vdo /dev/sda3 albserver online cpu=2,bio=4,ack=1,bioRotationInterval=64"
    assert vdo.data["VDOs"]["vdo1"]["VDO statistics"]["/dev/mapper/vdo1"]["KVDO module bios used"] == 74572

    assert vdo.get_slab_size_of_vol('vdo1') == '2G'

    assert vdo.volumns == ['vdo1', 'vdo2']

    assert vdo.get_physical_blocks_of_vol('vdo1') == 1835008
    assert vdo.get_overhead_used_of_vol('vdo1') == 787140
    assert vdo.get_physical_used_of_vol('vdo1') == 0

    assert vdo.get_logical_blocks_of_vol('vdo1') == 1046277
    assert vdo.get_logical_used_of_vol('vdo1') == 0

    assert vdo.get_logical_free_of_vol('vdo1') == 1046277
    assert vdo.get_physical_free_of_vol('vdo1') == 1047868

    assert vdo.get_logical_used_of_vol('vdo2') == 3
    assert vdo.get_physical_used_of_vol('vdo2') == 1


def test_vdo_status_documentation():
    """
    Here we test the examples in the documentation automatically using
    doctest.  We set up an environment which is similar to what a rule
    writer might see - a '/usr/bin/vdo status' command output that has
    been passed in as a parameter to the rule declaration.
    """
    env = {'vdo': VDOStatus(context_wrap(INPUT_STATUS_FULL))}
    failed, total = doctest.testmod(vdo_status, globs=env)
    assert failed == 0


def test_vdo_status_exp():
    """
    Here test the examples cause expections
    """
    with pytest.raises(ParseException) as sc1:
        VDOStatus(context_wrap(INPUT_EXP))
    assert "couldn't parse yaml" in str(sc1)


def test_vdo_status_exp3():
    """
    Here test the examples cause expections
    """
    with pytest.raises(KeyError) as sc1:
        vdo = VDOStatus(context_wrap(INPUT_STATUS_SIMPLE))
        vdo.get_physical_blocks_of_vol('vdo3')
    assert "No key(s) named" in str(sc1)



"""
Lssap - command ``/usr/sap/hostctrl/exe/lssap``
===============================================

This module provides processing for the output of the ``lssap`` command on
SAP systems. The spec handled by this command inlude::

    "lssap"                     : CommandSpec("/usr/sap/hostctrl/exe/lssap")

Class ``Lssap`` parses the output of the ``lssap`` command.  Sample
output of this command looks like::

 - lssap version 1.0 -
 ==========================================
   SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
   HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
   HA2|  22|       D22|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
   HA2|  50|       D50|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D50/exe
   HA2|  51|       D51|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D51/exe

Examples:
    >>> lssap.instances
    ['D16', 'D22', 'D50', 'D51']
    >>> lssap.version('D51')
    '749, patch 10, changelist 1698137'
    >>> lssap.is_hana()
    False
    >>> lssap.data[3]['Instance']
    'D51'
"""
from insights import parser, CommandParser
from insights.parsers import SkipException, ParseException, parse_delimited_table
from insights.specs import Specs


@parser(Specs.lssap)
class Lssap(CommandParser):
    """Class to parse ``lssap`` command output.

    Raises:
        SkipException: Nothing needs to be parsed.
        ParseException: Raised if any error occurs parsing the content.

    Attributes:
        data (list): List of dicts, where the keys in each dict are the column
            headers and each item in the list represents a SID.
        sid (list): List of the SIDs from the SID column.
        instances (list): List of instances running on the system.
        instance_types (list): List of instance types running on the system.
    """
    def parse_content(self, content):
        if not content:
            raise SkipException()

        self.data = []
        # remove lssap version and bar text from content
        clean_content = content[2:-1]
        if len(clean_content) > 0 and clean_content[0].lstrip().startswith("SID"):
            self.data = parse_delimited_table(clean_content, delim='|', header_delim=None)
        else:
            raise ParseException("Lssap: Unable to parse {0} line(s) of content: ({1})".format(len(content), content))

        self.sid = sorted(set(row["SID"] for row in self.data if "SID" in row))
        self.instances = [row["Instance"] for row in self.data if "Instance" in row]
        self.instance_types = sorted(set(inst.rstrip('1234567890') for inst in self.instances))

    def version(self, instance):
        """str: returns the Version column corresponding to the ``instance`` in
        Instance or ``None`` if ``instance`` is not found.
        """
        for row in self.data:
            if instance == row['Instance']:
                return row["Version"]

    def is_netweaver(self):
        """bool: Is any SAP NetWeaver instance detected?"""
        return 'D' in self.instance_types

    def is_hana(self):
        """bool: Is any SAP HANA instance detected?"""
        return 'HDB' in self.instance_types

    def is_ascs(self):
        """bool: Is any SAP System Central Services instance detected?"""
        return 'ASCS' in self.instance_types
